from rich_rst._vendor import docutils
import rich_rst._vendor.docutils.core
from rich.console import Console
from rich.text import Text

from rich_rst import RSTVisitor


def test_plain_definition_list_uses_term_style_branch():
    document = docutils.core.publish_doctree("term\n    definition\n")
    visitor = RSTVisitor(
        document,
        console=Console(force_terminal=True, record=True),
        code_theme="monokai",
        show_line_numbers=False,
        guess_lexer=True,
        default_lexer="python",
    )

    document.walkabout(visitor)

    assert len(visitor.renderables) == 1
    renderable = visitor.renderables[0]
    assert isinstance(renderable, Text)
    assert renderable.plain == "term\n    definition\n      "
    assert renderable.spans == []



"""Tests for the RestructuredText public API.

Covers: class instantiation, default options, all alias names,
custom options (code_theme, show_line_numbers, filename, sphinx_compat,
show_errors), and end-to-end rendering without exceptions.
"""
from rich.console import Console
from rich_rst import (
    RST,
    ReStructuredText,
    RestructuredText,
    RSTVisitor,
    reStructuredText,
)


# ── Alias names ───────────────────────────────────────────────────────────────

def test_rst_alias_is_restructuredtext():
    assert RST is RestructuredText


def test_restructuredtext_camelcase_alias():
    assert ReStructuredText is RestructuredText


def test_lowercase_alias():
    assert reStructuredText is RestructuredText


# ── Default option values ─────────────────────────────────────────────────────

def test_default_sphinx_compat_is_true():
    assert RestructuredText("text").sphinx_compat is True


def test_default_show_errors_is_true():
    assert RestructuredText("text").log_errors is True


def test_default_code_theme_is_monokai():
    assert RestructuredText("text").code_theme == "monokai"


def test_default_show_line_numbers_is_false():
    assert RestructuredText("text").show_line_numbers is False


def test_default_guess_lexer_is_false():
    assert RestructuredText("text").guess_lexer is False


def test_default_default_lexer_is_python():
    assert RestructuredText("text").default_lexer == "python"


def test_default_filename():
    assert RestructuredText("text").filename == "<rst-document>"


# ── Custom option values ──────────────────────────────────────────────────────

def test_custom_code_theme():
    rst = RestructuredText("text", code_theme="github-dark")
    assert rst.code_theme == "github-dark"


def test_show_line_numbers_enabled():
    rst = RestructuredText("text", show_line_numbers=True)
    assert rst.show_line_numbers is True


def test_custom_filename():
    rst = RestructuredText("text", filename="my_doc.rst")
    assert rst.filename == "my_doc.rst"


def test_sphinx_compat_disabled():
    rst = RestructuredText("text", sphinx_compat=False)
    assert rst.sphinx_compat is False


def test_show_errors_disabled():
    rst = RestructuredText("text", show_errors=False)
    assert rst.log_errors is False


def test_custom_default_lexer():
    rst = RestructuredText("text", default_lexer="bash")
    assert rst.default_lexer == "bash"


def test_guess_lexer_enabled():
    rst = RestructuredText("text", guess_lexer=True)
    assert rst.guess_lexer is True


# ── End-to-end rendering ──────────────────────────────────────────────────────

def test_render_plain_text_no_exception(render_text):
    out = render_text("Hello **world**.")
    assert "Hello" in out
    assert "world" in out


def test_render_empty_string_no_exception():
    console = Console(force_terminal=True, record=True, width=80)
    console.print(RestructuredText(""))
    # Should not raise


def test_render_complex_document_no_exception(render_text):
    rst = """\
Title
=====

A paragraph with **bold**, *italic*, and ``code``.

.. note::

   A note.

* item one
* item two

#. first
#. second
"""
    out = render_text(rst)
    assert "Title" in out
    assert "bold" in out


# ── Error handling ────────────────────────────────────────────────────────────

def test_show_errors_true_shows_system_message(render_text):
    # An unknown role triggers a system message
    out = render_text(
        ":my_unknown_role_xyz:`text`",
        show_errors=True,
        sphinx_compat=False,
    )
    assert "System Message" in out


def test_show_errors_false_hides_system_message(render_text):
    out = render_text(
        ":my_unknown_role_xyz:`text`",
        show_errors=False,
        sphinx_compat=False,
    )
    assert "System Message" not in out


def test_problematic_node_renders_as_error_panel(render_text):
    out = render_text(
        ":my_unknown_role_xyz:`text`",
        show_errors=True,
        sphinx_compat=False,
    )
    assert "Problematic Element" in out


def test_no_exception_on_system_message():
    """Rendering a document with system messages must never raise."""
    console = Console(force_terminal=True, record=True, width=80)
    try:
        console.print(
            RestructuredText(
                ":my_unknown_role_xyz:`text`",
                show_errors=True,
                sphinx_compat=False,
            )
        )
    except Exception as exc:
        raise AssertionError(f"Rendering raised an unexpected exception: {exc}") from exc



"""Comprehensive coverage tests for rich-rst.

This file contains tests targeting specific coverage gaps identified through coverage analysis.

Focus areas:
- Lexer guessing and fallback logic
- Section level calculation with nested sections  
- Reference and target node handling
- Field/docinfo variations (authors, organization, address, etc.)
- Definition list edge cases
- Complex nested list scenarios
- Block quote with/without attribution
- Line block nesting
- Topic and sidebar rendering
- Option list variations
- Abbreviations and acronyms
- Emphasis and strong emphasis variations
- Subscript and superscript handling
- Citations and footnotes
- Raw content elements
- Complex table and figure scenarios
- Inline markup combinations
- Generated and pending nodes
"""
import pytest
from rich.console import Console
from rich_rst import RestructuredText, RSTVisitor


# ══════════════════════════════════════════════════════════════════════════════
# LEXER AND CODE HANDLING
# ══════════════════════════════════════════════════════════════════════════════

def test_guess_lexer_with_unknown_language(render_text):
    """Test guessing lexer for code that doesn't match any known patterns."""
    rst = """\
.. code-block::

   this is just some random text
   not any specific language
   maybe looks like something
"""
    out = render_text(rst, guess_lexer=False)
    assert "python" in out  # Default lexer


def test_lexer_guess_fallback_to_default(render_text):
    """Test that when guess fails, it returns default lexer."""
    rst = """\
.. code-block::

   ξξξξξξ random unicode ξξξξξξ
"""
    out = render_text(rst, guess_lexer=True, default_lexer="python")
    assert "python" in out or "random unicode" in out


def test_code_block_with_explicit_format(render_text):
    """Test code block with explicit format specification."""
    rst = """\
.. code-block:: javascript

   console.log("hello world");
"""
    out = render_text(rst)
    assert "console.log" in out


def test_code_block_with_class_syntax(render_text):
    """Test code block using class syntax for language."""
    rst = """\
.. code-block::
   :class: language-rust

   fn main() {
       println!("Hello");
   }
"""
    out = render_text(rst)
    assert "fn main" in out


def test_lexer_with_aliases(render_text):
    """Test lexer that has aliases (normal case)."""
    rst = """\
.. code-block:: python3

   x = 1
"""
    out = render_text(rst, guess_lexer=False)
    assert len(out) > 0


def test_render_with_line_numbers(render_text):
    """Test rendering with line numbers enabled."""
    rst = """\
.. code-block:: python

   def hello():
       return "world"
"""
    out = render_text(rst, show_line_numbers=True)
    assert len(out) > 0


def test_render_with_custom_code_theme(render_text):
    """Test rendering with different code theme."""
    rst = """\
.. code-block:: python

   x = 42
"""
    out = render_text(rst, code_theme="github-dark")
    assert len(out) > 0


# ══════════════════════════════════════════════════════════════════════════════
# SECTIONS AND STRUCTURE
# ══════════════════════════════════════════════════════════════════════════════

def test_deeply_nested_sections(render_text):
    """Test multiple levels of section nesting."""
    rst = """\
Level 1
=======

Some text here.

Level 2
-------

More text.

Level 3
^^^^^^^

Even deeper.

Level 4
"""""""

Deepest level.
"""
    out = render_text(rst)
    assert "Level 1" in out
    assert "Level 2" in out
    assert "Level 3" in out
    assert "Level 4" in out


def test_multiple_sections_same_level(render_text):
    """Test multiple sections at the same level."""
    rst = """\
First Section
=============

Content here.

Second Section
==============

Different content.

Third Section
=============

More content.
"""
    out = render_text(rst)
    assert "First Section" in out
    assert "Second Section" in out
    assert "Third Section" in out


# ══════════════════════════════════════════════════════════════════════════════
# REFERENCES AND TARGETS
# ══════════════════════════════════════════════════════════════════════════════

def test_reference_with_inline_image(render_text):
    """Test that reference with image child is properly skipped."""
    rst = """\
`Link with image <http://example.com>`_

.. image:: /some/image.png
   :target: http://example.com
"""
    out = render_text(rst)
    assert len(out) > 0


def test_reference_resolution_via_target(render_text):
    """Test reference name resolution with explicit target."""
    rst = """\
See the `introduction`_ document.

.. _introduction: https://example.com/intro
"""
    out = render_text(rst)
    assert "introduction" in out


def test_anonymous_target(render_text):
    """Test anonymous hyperlink targets."""
    rst = """\
This is an __ anonymous link.

__ https://example.com
"""
    out = render_text(rst)
    assert "anonymous" in out


def test_multiple_targets_same_name(render_text):
    """Test handling of duplicate target names."""
    rst = """\
First `reference`_.

.. _reference: https://example.com/1

Second reference text.
"""
    out = render_text(rst)
    assert "reference" in out


def test_reference_without_refuri_or_refname(render_text):
    """Test reference that has neither refuri nor refname."""
    rst = "Some text with regular link reference."
    out = render_text(rst)
    assert "text" in out


def test_multiple_inline_references_sequence(render_text):
    """Test multiple references in sequence."""
    rst = """\
Check `link1`_ and `link2`_ and `link3`_.

.. _link1: http://example.com/1
.. _link2: http://example.com/2  
.. _link3: http://example.com/3
"""
    out = render_text(rst)
    assert "link" in out


def test_title_reference_appended_to_text(render_text):
    """Test title reference appended to text."""
    rst = """\
See the `Title`_ document for more.
"""
    out = render_text(rst)
    assert "Title" in out


# ══════════════════════════════════════════════════════════════════════════════
# IMAGES AND FIGURES
# ══════════════════════════════════════════════════════════════════════════════

def test_image_with_alt_attribute(render_text):
    """Test image with alt text."""
    rst = """\
.. image:: /path/image.png
   :alt: Alternative text
"""
    out = render_text(rst)
    assert len(out) > 0


def test_image_with_target_attribute(render_text):
    """Test image with target link."""
    rst = """\
.. image:: /path/image.png
   :target: http://example.com
"""
    out = render_text(rst)
    assert len(out) > 0


def test_image_with_alt_and_target(render_text):
    """Test image with both alt and target."""
    rst = """\
.. image:: /path/image.png
   :alt: Image description
   :target: http://example.com
"""
    out = render_text(rst)
    assert len(out) > 0


def test_figure_without_image(render_text):
    """Test figure element without image (unusual case)."""
    rst = """\
.. figure::

   Just a caption, no image.
"""
    out = render_text(rst)
    assert len(out) > 0


def test_figure_with_reference_target(render_text):
    """Test figure with target inside reference."""
    rst = """\
.. figure:: /path/image.png
   :target: http://example.com

   Caption text
   Legend line 1
   Legend line 2
"""
    out = render_text(rst)
    assert len(out) > 0


def test_figure_with_caption(render_text):
    """Test figure with caption."""
    rst = """\
.. figure:: /path/to/image.png

   This is the caption.
"""
    out = render_text(rst)
    assert len(out) > 0


def test_figure_with_caption_and_legend(render_text):
    """Test figure with caption and legend."""
    rst = """\
.. figure:: /path/to/image.png

   Figure caption text.

   Legend text
   more legend.
"""
    out = render_text(rst)
    assert len(out) > 0


def test_linked_image_with_complex_attributes(render_text):
    """Test image with multiple attributes in a link."""
    rst = """\
`Image Link <http://example.com>`_

.. image:: /path/image.png
   :alt: Image description
   :target: http://example.com
   :width: 200
   :height: 100
"""
    out = render_text(rst)
    assert len(out) > 0


# ══════════════════════════════════════════════════════════════════════════════
# FIELD LISTS AND DOCINFO
# ══════════════════════════════════════════════════════════════════════════════

def test_multiple_authors_docinfo(render_text):
    """Test docinfo with multiple author elements."""
    rst = """\
:Author1: Alice
:Author2: Bob
:Organization: ACME Corp
:Contact: contact@example.com
:Version: 1.0
:Revision: 1.1
:Status: Draft
:Date: 2024-01-01
:Copyright: 2024 ACME
"""
    out = render_text(rst)
    assert "Author" in out
    assert "Organization" in out


def test_docinfo_single_author(render_text):
    """Test docinfo with single author."""
    rst = """\
:Author: Charlie
:Date: 2024-06-15
"""
    out = render_text(rst)
    assert "Author" in out
    assert "Charlie" in out


def test_multiple_field_merging(render_text):
    """Test that multiple field blocks merge into same table."""
    rst = """\
:Author: Alice
:Date: 2024-01-01

Some paragraph.

:Version: 1.0
:Status: Draft
"""
    out = render_text(rst)
    assert "Author" in out
    assert "Version" in out


def test_field_list_standalone(render_text):
    """Test field list outside of docinfo."""
    rst = """\
Content before.

:Field1: Value1
:Field2: Value2
:Field3: Value3

Content after.
"""
    out = render_text(rst)
    assert "Field1" in out
    assert "Value1" in out


def test_address_docinfo(render_text):
    """Test address field in docinfo."""
    rst = """\
:Address:
   123 Main Street
   Springfield, IL 62701
:Contact: info@example.com
"""
    out = render_text(rst)
    assert "Address" in out or "123" in out


# ══════════════════════════════════════════════════════════════════════════════
# DEFINITION LISTS
# ══════════════════════════════════════════════════════════════════════════════

def test_definition_list_with_classifier(render_text):
    """Test definition list with classifier."""
    rst = """\
term
   classified : classifier
   The definition.
"""
    out = render_text(rst)
    assert "term" in out
    assert "definition" in out or "classified" in out


def test_definition_list_without_classifier(render_text):
    """Test definition list without classifier."""
    rst = """\
term
   The definition without classifier.

another
   Another definition.
"""
    out = render_text(rst)
    assert "term" in out
    assert "definition" in out


def test_definition_list_with_nested_content(render_text):
    """Test definition list containing nested lists."""
    rst = """\
Python
   A programming language.

   * Feature 1
   * Feature 2

   Code example::

      print("hello")
"""
    out = render_text(rst)
    assert "Python" in out
    assert "Feature" in out


def test_definition_list_multiple_lines(render_text):
    """Test definition list with multi-line content."""
    rst = """\
item
   First line of definition.
   Second line of definition.
   Third line of definition.
"""
    out = render_text(rst)
    assert "item" in out
    assert "definition" in out


def test_definition_list_three_parts(render_text):
    """Test definition list with term, classifier, and definition."""
    rst = """\
term : classifier
   The definition of the term with classifier.
"""
    out = render_text(rst)
    assert len(out) > 0


def test_complex_definition_list_mixed(render_text):
    """Test complex definition list mixing various formats."""
    rst = """\
term1
   definition1

term2 : classifier
   definition2

term3
   def line 1
   def line 2
"""
    out = render_text(rst)
    assert "term1" in out or "definition" in out


def test_definition_list_nested_lists_and_code(render_text):
    """Test definition list with nested lists and code blocks."""
    rst = """\
Term with complex definition
   This definition has multiple components.
   
   Related items:
   
   1. First related
   2. Second related
   
   Code example::
   
      x = 1
      y = 2
      
   Final note about this term.
"""
    out = render_text(rst)
    assert "Term" in out or "definition" in out


# ══════════════════════════════════════════════════════════════════════════════
# OPTION LISTS
# ══════════════════════════════════════════════════════════════════════════════

def test_option_list_with_arguments(render_text):
    """Test option list with arguments."""
    rst = """\
-a <arg>
   Description of -a option
-b, --beta VALUE
   Description of beta option
-c
   Boolean flag
"""
    out = render_text(rst)
    assert len(out) > 0


def test_option_list_single_option(render_text):
    """Test option list with single option."""
    rst = """\
--verbose
   Enable verbose output
"""
    out = render_text(rst)
    assert len(out) > 0


def test_complex_option_list(render_text):
    """Test complex option list with various formats."""
    rst = """\
Command Options
===============

-h, --help
   Show this help message
-v, --verbose
   Enable verbose output
-o FILE, --output FILE
   Write output to FILE
-q, --quiet
   Suppress output
--config=FILE
   Use configuration from FILE
"""
    out = render_text(rst)
    assert len(out) > 0


# ══════════════════════════════════════════════════════════════════════════════
# BLOCK QUOTES
# ══════════════════════════════════════════════════════════════════════════════

def test_block_quote_with_attribution(render_text):
    """Test block quote with attribution."""
    rst = """\
   This is a great quote.
   It spans multiple lines.

   — Famous Person
"""
    out = render_text(rst)
    assert "quote" in out


def test_block_quote_multiple_paragraphs(render_text):
    """Test block quote with multiple paragraphs."""
    rst = """\
   First paragraph of quote.

   Second paragraph continues.

   Third paragraph concludes.
"""
    out = render_text(rst)
    assert "paragraph" in out or "quote" in out


def test_block_quote_no_attribution(render_text):
    """Test block quote without attribution."""
    rst = """\
   A simple quote.
   No attribution here.
"""
    out = render_text(rst)
    assert "quote" in out


def test_block_quote_single_paragraph(render_text):
    """Test block quote with exactly one paragraph."""
    rst = """\
   Single paragraph quote.
"""
    out = render_text(rst)
    assert "quote" in out or "paragraph" in out


def test_block_quote_many_paragraphs(render_text):
    """Test block quote with many paragraphs."""
    rst = """\
   Para 1
     
   Para 2
     
   Para 3
     
   Para 4
     
   Para 5
"""
    out = render_text(rst)
    assert len(out) > 0


def test_nested_content_in_block_quote(render_text):
    """Test block quote containing various nested content."""
    rst = """\
   Block quote with multiple paragraphs.
   
   First paragraph here.
   
   Second paragraph with **bold** and *italic* text.

   — Attribution
"""
    out = render_text(rst)
    assert "quote" in out or "paragraph" in out


# ══════════════════════════════════════════════════════════════════════════════
# LINE BLOCKS
# ══════════════════════════════════════════════════════════════════════════════

def test_line_block_nested(render_text):
    """Test nested line blocks."""
    rst = """\
| Line 1
| Line 2
|
|     Indented line 3
|     Indented line 4
| Line 5
"""
    out = render_text(rst)
    assert "Line 1" in out
    assert "Line 5" in out


def test_line_block_deeply_nested(render_text):
    """Test deeply nested line blocks."""
    rst = """\
| Level 1-1
|
|     Level 2-1
|     Level 2-2
|
|         Level 3-1
|
|     Back to Level 2
|
| Back to Level 1
"""
    out = render_text(rst)
    assert "Level 1-1" in out


def test_empty_line_block(render_text):
    """Test line block that's essentially empty."""
    rst = """\
| Just one line
"""
    out = render_text(rst)
    assert "Just one line" in out


def test_line_block_single_line(render_text):
    """Test line block with single line."""
    rst = """\
| Single line
"""
    out = render_text(rst)
    assert "Single line" in out


def test_line_block_many_levels(render_text):
    """Test line block with many indentation levels."""
    rst = """\
| L1
|     L2
|         L3
|             L4
|                 L5
| Back to L1
"""
    out = render_text(rst)
    assert "L1" in out or "L5" in out


# ══════════════════════════════════════════════════════════════════════════════
# ADMONITIONS
# ══════════════════════════════════════════════════════════════════════════════

def test_empty_body_rendering(render_text):
    """Test that empty body lists render without crashing."""
    rst = """\
.. note::

   Minimal content.
"""
    out = render_text(rst)
    assert "Note" in out or "Minimal" in out


def test_admonition_with_custom_title(render_text):
    """Test generic admonition with custom title node."""
    rst = """\
.. admonition:: Custom Title

   Content of custom admonition.
"""
    out = render_text(rst)
    assert "Custom Title" in out


def test_admonition_with_nested_lists_and_code(render_text):
    """Test admonition containing nested lists and code."""
    rst = """\
.. warning::

   This warning contains important information.
   
   Key points:
   
   * Point one
   * Point two
   
   Code to avoid::
   
      dangerous_function()
      
   Always use::
   
      safe_function()
"""
    out = render_text(rst)
    assert "warning" in out.lower() or "Warning" in out


# ══════════════════════════════════════════════════════════════════════════════
# TOPICS AND SIDEBARS
# ══════════════════════════════════════════════════════════════════════════════

def test_topic_with_title(render_text):
    """Test topic with explicit title."""
    rst = """\
.. topic:: Important Topic

   This is the content of the topic.
   It can have multiple paragraphs.
"""
    out = render_text(rst)
    assert "Important Topic" in out


def test_topic_no_title(render_text):
    """Test topic without title."""
    rst = """\
.. topic::

   Content here.
"""
    out = render_text(rst)
    assert "Content" in out


def test_sidebar_with_title_and_subtitle(render_text):
    """Test sidebar with title and subtitle."""
    rst = """\
.. sidebar:: Sidebar Title
   :subtitle: Subtitle

   Sidebar content goes here.
"""
    out = render_text(rst)
    assert "Sidebar" in out


def test_sidebar_title_only(render_text):
    """Test sidebar with only title."""
    rst = """\
.. sidebar:: My Sidebar

   Just the content.
"""
    out = render_text(rst)
    assert "Sidebar" in out or "content" in out


def test_sidebar_with_subtitle_and_lists(render_text):
    """Test sidebar with subtitle and nested lists."""
    rst = """\
.. sidebar:: Sidebar Title
   :subtitle: Interesting Subtitle
   
   Sidebar content with bullet list:
   
   * Item 1
   * Item 2
   * Item 3
"""
    out = render_text(rst)
    assert len(out) > 0


def test_topic_with_lists_and_code(render_text):
    """Test topic element with nested lists and code."""
    rst = """\
.. topic:: Important Topic

   This topic covers:
   
   1. First concept
   2. Second concept
   
   Example::
   
      example_code()
"""
    out = render_text(rst)
    assert "topic" in out.lower() or "Topic" in out


# ══════════════════════════════════════════════════════════════════════════════
# TRANSITIONS AND SEPARATORS
# ══════════════════════════════════════════════════════════════════════════════

def test_transition_between_sections(render_text):
    """Test transition/horizontal rule."""
    rst = """\
Section 1
=========

Content here.

----

Section 2
=========

More content.
"""
    out = render_text(rst)
    assert "Section 1" in out
    assert "Section 2" in out


def test_multiple_transitions(render_text):
    """Test multiple transitions."""
    rst = """\
First block.

----

Second block.

----

Third block.
"""
    out = render_text(rst)
    assert "First" in out
    assert "Second" in out


# ══════════════════════════════════════════════════════════════════════════════
# DOCTEST AND MATH BLOCKS
# ══════════════════════════════════════════════════════════════════════════════

def test_doctest_block(render_text):
    """Test doctest block rendering."""
    rst = """\
.. doctest::

   >>> x = 1 + 2
   >>> print(x)
   3
"""
    out = render_text(rst)
    assert len(out) > 0


def test_doctest_multiple_examples(render_text):
    """Test multiple doctest examples."""
    rst = """\
.. doctest::

   >>> list(range(3))
   [0, 1, 2]
   >>> dict(a=1, b=2)
   {'a': 1, 'b': 2}
"""
    out = render_text(rst)
    assert len(out) > 0


def test_doctest_standalone(render_text):
    """Test doctest directive standalone."""
    rst = """\
.. doctest::

   >>> print("test")
   test
"""
    out = render_text(rst)
    assert len(out) > 0


def test_math_block_inline(render_text):
    """Test inline math rendering."""
    rst = """\
Some text :math:`E = mc^2` more text.
"""
    out = render_text(rst)
    assert "text" in out


def test_math_block_display(render_text):
    """Test display math block."""
    rst = """\
.. math::

   E = mc^2
"""
    out = render_text(rst)
    assert len(out) > 0


def test_math_standalone(render_text):
    """Test math directive standalone."""
    rst = """\
.. math::

   \\frac{a}{b}
"""
    out = render_text(rst)
    assert len(out) > 0


# ══════════════════════════════════════════════════════════════════════════════
# ABBREVIATIONS AND ACRONYMS
# ══════════════════════════════════════════════════════════════════════════════

def test_abbreviation_with_explanation(render_text):
    """Test abbreviation with explanation."""
    rst = """\
The |abbr| (abbreviation) is common.

.. |abbr| abbreviation:: An abbreviation
"""
    out = render_text(rst)
    assert len(out) > 0


def test_acronym_with_explanation(render_text):
    """Test acronym with explanation."""
    rst = """\
The |acr| (acronym) is used here.

.. |acr| acronym:: An Acronym Code
"""
    out = render_text(rst)
    assert len(out) > 0


def test_abbreviation_inline_full_markup(render_text):
    """Test abbreviation with full inline markup."""
    rst = """\
Use |HTML| in markup.

.. |HTML| abbreviation:: HyperText Markup Language
"""
    out = render_text(rst)
    assert len(out) > 0


# ══════════════════════════════════════════════════════════════════════════════
# EMPHASIS AND STRONG TEXT
# ══════════════════════════════════════════════════════════════════════════════

def test_emphasis_appended_to_previous_text(render_text):
    """Test emphasis appended to previous text element."""
    rst = """\
This is regular text *and this is emphasized*.
"""
    out = render_text(rst)
    assert "regular text" in out
    assert "emphasized" in out


def test_strong_appended_to_previous_text(render_text):
    """Test strong emphasis appended to previous text."""
    rst = """\
Regular **and this is bold**.
"""
    out = render_text(rst)
    assert "Regular" in out
    assert "bold" in out


def test_emphasis_first_element(render_text):
    """Test emphasis as first element."""
    rst = """\
*Starts with emphasis* in a paragraph.
"""
    out = render_text(rst)
    assert "emphasis" in out


# ══════════════════════════════════════════════════════════════════════════════
# SUBSCRIPT AND SUPERSCRIPT
# ══════════════════════════════════════════════════════════════════════════════

def test_subscript_appended_to_text(render_text):
    """Test subscript appended to existing text."""
    rst = """\
H\ :sub:`2`\ O is water.
"""
    out = render_text(rst)
    assert len(out) > 0


def test_superscript_appended_to_text(render_text):
    """Test superscript appended to existing text."""
    rst = """\
E=mc\ :sup:`2`\ is Einstein's formula.
"""
    out = render_text(rst)
    assert len(out) > 0


def test_subscript_first_element(render_text):
    """Test subscript as first element."""
    rst = """\
:sub:`subscript` at the beginning.
"""
    out = render_text(rst)
    assert len(out) > 0


# ══════════════════════════════════════════════════════════════════════════════
# LISTS: BULLET AND ENUMERATED
# ══════════════════════════════════════════════════════════════════════════════

def test_bullet_list_with_nested_enumerated_list(render_text):
    """Test bullet list containing enumerated list."""
    rst = """\
* Bullet 1

  1. Numbered 1.1
  2. Numbered 1.2

* Bullet 2

  1. Numbered 2.1
  2. Numbered 2.2
"""
    out = render_text(rst)
    assert "Bullet" in out


def test_enumerated_list_with_nested_bullet_list(render_text):
    """Test enumerated list containing bullet list."""
    rst = """\
1. First item

   * Nested bullet A
   * Nested bullet B

2. Second item

   * Nested bullet C
"""
    out = render_text(rst)
    assert "First" in out


def test_triple_nested_lists(render_text):
    """Test three levels of list nesting."""
    rst = """\
* Level 1-1
  
  * Level 2-1
    
    * Level 3-1
    * Level 3-2
  
  * Level 2-2

* Level 1-2
"""
    out = render_text(rst)
    assert len(out) > 0


def test_list_with_code_block(render_text):
    """Test list containing code block."""
    rst = """\
* First item
* Second item with code::

      def hello():
          return "world"

* Third item
"""
    out = render_text(rst)
    assert "Second" in out or "hello" in out


def test_bullet_list_with_only_code_blocks(render_text):
    """Test bullet list containing only code blocks."""
    rst = """\
* ::

     code block 1

* ::

     code block 2
"""
    out = render_text(rst)
    assert len(out) > 0


def test_enumerated_list_with_only_code_blocks(render_text):
    """Test enumerated list containing only code blocks."""
    rst = """\
1. ::

      code block here
    
2. ::

      another code block
"""
    out = render_text(rst)
    assert len(out) > 0


def test_very_deep_enumerated_list_nesting(render_text):
    """Test very deeply nested enumerated lists."""
    rst = """\
1. Item 1

   1. Item 1.1
   
      1. Item 1.1.1
      
         1. Item 1.1.1.1
"""
    out = render_text(rst)
    assert "Item" in out


# ══════════════════════════════════════════════════════════════════════════════
# CITATIONS AND FOOTNOTES
# ══════════════════════════════════════════════════════════════════════════════

def test_citation_reference(render_text):
    """Test citation reference."""
    rst = """\
Some text [CIT2024]_.

.. [CIT2024] A citation.
"""
    out = render_text(rst)
    assert len(out) > 0


def test_footnote_reference(render_text):
    """Test footnote reference."""
    rst = """\
Some text [#]_.

.. [#] A footnote.
"""
    out = render_text(rst)
    assert len(out) > 0


def test_multiple_footnotes(render_text):
    """Test multiple footnotes."""
    rst = """\
First [1]_ and second [2]_.

.. [1] First note.
.. [2] Second note.
"""
    out = render_text(rst)
    assert len(out) > 0


def test_citation_reference_appended_to_text(render_text):
    """Test citation reference appended to existing text element."""
    rst = """\
See the cited work [Ref2024]_.

.. [Ref2024] A citation.
"""
    out = render_text(rst)
    assert len(out) > 0


def test_footnote_reference_appended_to_text(render_text):
    """Test footnote reference appended to existing text element."""
    rst = """\
This is text [1]_ with a footnote.

.. [1] Footnote text.
"""
    out = render_text(rst)
    assert len(out) > 0


def test_numbered_footnote(render_text):
    """Test explicitly numbered footnote."""
    rst = """\
Text [1]_.

.. [1] First footnote.
"""
    out = render_text(rst)
    assert len(out) > 0


def test_auto_numbered_footnote(render_text):
    """Test auto-numbered footnote."""
    rst = """\
Text [#]_.

.. [#] Auto-numbered footnote.
"""
    out = render_text(rst)
    assert len(out) > 0


def test_labeled_footnote(render_text):
    """Test labeled footnote."""
    rst = """\
Text [note]_.

.. [note] A labeled footnote.
"""
    out = render_text(rst)
    assert len(out) > 0


def test_citation_block(render_text):
    """Test citation block."""
    rst = """\
Reference [Book2024]_.

.. [Book2024] A Book Title. Published 2024.
"""
    out = render_text(rst)
    assert len(out) > 0


# ══════════════════════════════════════════════════════════════════════════════
# RAW CONTENT
# ══════════════════════════════════════════════════════════════════════════════

def test_raw_html_element(render_text):
    """Test raw HTML element."""
    rst = """\
.. raw:: html

   <div>This is raw HTML</div>
"""
    out = render_text(rst)
    assert len(out) > 0


def test_raw_latex_element(render_text):
    """Test raw LaTeX element."""
    rst = """\
.. raw:: latex

   \\textbf{Bold text}
"""
    out = render_text(rst)
    assert len(out) > 0


def test_raw_text_format(render_text):
    """Test raw text format."""
    rst = """\
.. raw:: text

   This is raw text content.
"""
    out = render_text(rst)
    assert len(out) > 0


def test_raw_with_special_chars(render_text):
    """Test raw content with special characters."""
    rst = """\
.. raw:: html

   <span class="special">&nbsp;&copy;&reg;</span>
"""
    out = render_text(rst)
    assert len(out) > 0


def test_raw_content_all_formats(render_text):
    """Test raw directive with different formats."""
    rst = """\
.. raw:: html

   <p>This is HTML</p>

.. raw:: latex

   \\textbf{Bold}
   
.. raw:: rst

   **Restructured** text
"""
    out = render_text(rst)
    assert len(out) > 0


# ══════════════════════════════════════════════════════════════════════════════
# TABLES
# ══════════════════════════════════════════════════════════════════════════════

def test_table_with_headers(render_text):
    """Test table with header row."""
    rst = """\
=====  =====
Col1   Col2
=====  =====
A      B
C      D
=====  =====
"""
    out = render_text(rst)
    assert "Col1" in out or "A" in out


def test_table_complex(render_text):
    """Test complex table."""
    rst = """\
+-------+-------+
| A     | B     |
+=======+=======+
| 1     | 2     |
+-------+-------+
| 3     | 4     |
+-------+-------+
"""
    out = render_text(rst)
    assert len(out) > 0


def test_table_with_cell_content(render_text):
    """Test table with various cell content."""
    rst = """\
+----------+----------+
| Cell 1   | Cell 2   |
+----------+----------+
| Content1 | Content2 |
+----------+----------+
"""
    out = render_text(rst)
    assert "Cell" in out or "Content" in out


def test_table_with_multiple_rows_and_columns(render_text):
    """Test complex table with many rows and columns."""
    rst = """\
+---------+---------+---------+
| Header1 | Header2 | Header3 |
+=========+=========+=========+
| Cell11  | Cell12  | Cell13  |
+---------+---------+---------+
| Cell21  | Cell22  | Cell23  |
+---------+---------+---------+
| Cell31  | Cell32  | Cell33  |
+---------+---------+---------+
"""
    out = render_text(rst)
    assert "Header" in out or "Cell" in out


def test_simple_table_format(render_text):
    """Test simple table format."""
    rst = """\
Simple Table
============  ============  ============
     A              B             C
============  ============  ============
    1              2             3
============  ============  ============
"""
    out = render_text(rst)
    assert len(out) > 0


def test_grid_table(render_text):
    """Test grid table format."""
    rst = """\
+---+---+
| A | B |
+===+===+
| 1 | 2 |
+---+---+
"""
    out = render_text(rst)
    assert len(out) > 0


# ══════════════════════════════════════════════════════════════════════════════
# INLINE CODE AND LITERALS
# ══════════════════════════════════════════════════════════════════════════════

def test_inline_literal_appended_to_text(render_text):
    """Test inline code appended to text."""
    rst = """\
Use the ``code`` variable in your script.
"""
    out = render_text(rst)
    assert "code" in out or "Use" in out


# ══════════════════════════════════════════════════════════════════════════════
# HEADERS AND FOOTERS
# ══════════════════════════════════════════════════════════════════════════════

def test_header_element(render_text):
    """Test document header element."""
    rst = """\
.. header:: This is a header
"""
    out = render_text(rst)
    assert len(out) >= 0


def test_footer_element(render_text):
    """Test document footer element."""
    rst = """\
.. footer:: Page ###
"""
    out = render_text(rst)
    assert len(out) >= 0


# ══════════════════════════════════════════════════════════════════════════════
# SYSTEM MESSAGES AND ERROR HANDLING
# ══════════════════════════════════════════════════════════════════════════════

def test_paragraph_containing_system_message(render_text):
    """Test paragraph that contains a system message."""
    rst = """\
Before error :unknown_role:`content` after error.
"""
    out = render_text(rst, show_errors=True, sphinx_compat=False)
    assert "System Message" in out


# ══════════════════════════════════════════════════════════════════════════════
# COMPREHENSIVE DOCUMENTS
# ══════════════════════════════════════════════════════════════════════════════

def test_complex_mixed_content(render_text):
    """Test complex document with mixed content types."""
    rst = """\
Main Title
==========

:Author: Test Author
:Date: 2024-06-15

Introduction paragraph with **bold** and *italic* text.

.. note::
   This is a note with more content.

Definition List
===============

term one
   Definition of term one with ``code``.
   
   * Nested bullet
   * Another bullet

term two
   Definition of term two.

Code Block Example
==================

.. code-block:: python
   :linenos:

   def example():
       return "Hello, World!"

Section with References
=======================

See `Google`_ for search.

.. _Google: https://google.com

----

Final Section
=============

Ending content.
"""
    out = render_text(rst)
    assert "Main Title" in out
    assert "Author" in out
    assert "Hello" in out


def test_rst_with_all_inline_markup(render_text):
    """Test RST with all inline markup types combined."""
    rst = """\
Inline Markup Test
==================

Text with **bold**, *italic*, **_bold italic_** and ``literal`` all together.

Also includes :sub:`subscript`, :sup:`superscript` and references to `some target`_.

.. _some target: http://example.com

Line with `emphasis`_, **strong**, and ``code`` in one go.
"""
    out = render_text(rst)
    assert "Markup" in out


def test_mixed_formatting_and_elements(render_text):
    """Test document with mixed formatting throughout."""
    rst = """\
Mixed Content
=============

Some text with `links <http://example.com>`_, **bold**, *italics*.

.. note:: Important note

   With nested **formatting**.

A paragraph before code::

   code block

After code, regular text continues.

* List item one

  With nested paragraph.

* List item two
"""
    out = render_text(rst)
    assert "Mixed" in out


def test_long_document_rendering(render_text):
    """Test rendering of a longer document to ensure it completes."""
    sections = []
    for i in range(5):
        sections.append(f"""\
Section {i}
{'-' * (10 + len(str(i)))}

Content for section {i}.

* Item 1 for section {i}
* Item 2 for section {i}

Example::

   code_{i}()
""")
    rst = "\n\n".join(sections)
    out = render_text(rst)
    assert "Section" in out


def test_entire_document_with_all_element_types(render_text):
    """Test complete document with as many element types as possible."""
    rst = """\
Complete Document
=================

:Author: Test
:Date: 2024-06-15

.. note:: A note

Introduction with |sub| and |sup|.

.. |sub| replace:: subscript-like
.. |sup| replace:: superscript-like

Bullet List
-----------

* Item 1
* Item 2

  * Nested

Enum List
---------

1. First
2. Second

Definition
----------

term
   definition

Quote
-----

   A quoted passage here.

Code::

   code sample

Raw
---

.. raw:: html

   <div>HTML</div>

Footer Test
-----------

.. footer:: Footer text
"""
    out = render_text(rst)
    assert "Complete Document" in out


def test_complex_comprehensive_document(render_text):
    """Test very comprehensive document using all major features."""
    rst = """\
Full Documentation
==================

:Author: John Doe
:Date: 2024-06-15
:Version: 2.0
:Status: Final

Abstract
--------

This is a comprehensive example covering multiple features.

.. note::
   An informational note.

Introduction
============

Section with **bold**, *italic*, and ``monospace`` text.

Features List
=============

* Feature One
* Feature Two

  * Sub-feature Alpha
  * Sub-feature Beta
  
* Feature Three

Numbered Items
==============

1. First item
2. Second item

   i. Sub-item 1
   ii. Sub-item 2
   
3. Third item

Glossary
========

Python
   A programming language.
   
   * Fast development
   * Easy to learn

RST
   ReStructuredText markup language.

Examples
========

Code example::

   def hello():
       return "world"

Another section
===============

| Line block example
| Second line here
|
|     Indented content
|
| Back to regular

Quotation
---------

   "The best way to predict the future is to invent it."
   
   — Alan Kay

Reference
---------

See the `Python docs`_ for more.

.. _Python docs: https://python.org

.. footer:: Page footer text
"""
    out = render_text(rst)
    assert "Full Documentation" in out


# ══════════════════════════════════════════════════════════════════════════════
# CONFIGURATION OPTIONS
# ══════════════════════════════════════════════════════════════════════════════

def test_rendering_with_errors_disabled(render_text):
    """Test rendering with error display disabled."""
    rst = """\
Some text with :unknown:`unknown role`.
"""
    out = render_text(rst, show_errors=False, sphinx_compat=False)
    assert "System Message" not in out


def test_rendering_without_sphinx_compat(render_text):
    """Test rendering without Sphinx compatibility."""
    rst = """\
Normal RST content.
"""
    out = render_text(rst, sphinx_compat=False)
    assert len(out) > 0


def test_render_all_rst_roles(render_text):
    """Test rendering with various RST roles."""
    rst = """\
Text with :emphasis:`emphasis`, :strong:`strong`, and :literal:`literal`.

Also :ref:`reference` and :doc:`document`.
"""
    out = render_text(rst)
    assert len(out) > 0


def test_render_with_syntax_error(render_text):
    """Test rendering malformed RST."""
    rst = """\
Unclosed ``literal

This should still render.
"""
    out = render_text(rst, show_errors=True)
    assert len(out) > 0


def test_very_long_line(render_text):
    """Test rendering with very long line."""
    long_text = "word " * 100
    rst = f"This is a very long line:\n\n{long_text}"
    out = render_text(rst)
    assert len(out) > 0


def test_many_nested_elements(render_text):
    """Test document with many nested elements."""
    rst = """\
Title
=====

Multiple inline styles: **bold *and italic* and ``code``** text.

Nested lists:

1. One

   * Alpha
   * Beta
   
     - i
     - ii

2. Two

   a. A
   b. B
"""
    out = render_text(rst)
    assert len(out) > 0


def test_empty_document(render_text):
    """Test rendering completely empty document."""
    rst = ""
    out = render_text(rst)
    assert len(out) >= 0


def test_minimal_valid_document(render_text):
    """Test minimal valid document."""
    rst = "Simple text."
    out = render_text(rst)
    assert "Simple text" in out


def test_direct_api_usage(render_text):
    """Test direct API usage through render_text with various options."""
    rst = "Text"
    out = render_text(
        rst,
        code_theme="vim",
        show_line_numbers=False,
        guess_lexer=False,
        default_lexer="bash",
        sphinx_compat=True,
        show_errors=True
    )
    assert "Text" in out


# ══════════════════════════════════════════════════════════════════════════════
# MISCELLANEOUS EDGE CASES
# ══════════════════════════════════════════════════════════════════════════════

def test_generated_node_handling(render_text):
    """Test generated nodes (typically auto-generated content)."""
    rst = "Some regular content."
    out = render_text(rst)
    assert "content" in out



# -*- coding: utf-8 -*-

"""
reStructuredText parser for rich

Initial few lines gotten from: https://github.com/willmcgugan/rich/discussions/1263#discussioncomment-808898
There are a lot of improvements are added by me
"""
from io import StringIO
from html.parser import HTMLParser
from typing import Optional, Union

# Imports from rich_rst._vendor.docutils package for the parsing
from rich_rst._vendor import docutils
import rich_rst._vendor.docutils.core
import rich_rst._vendor.docutils.frontend
import rich_rst._vendor.docutils.io
import rich_rst._vendor.docutils.nodes
import rich_rst._vendor.docutils.parsers.rst
import rich_rst._vendor.docutils.parsers.rst.directives
import rich_rst._vendor.docutils.utils

# Imports from the rich package for the printing
import rich
from rich import box
from rich.align import Align
from rich.console import Console, ConsoleOptions, RenderResult, NewLine, Group
from rich.jupyter import JupyterMixin
from rich.panel import Panel
from rich.style import Style
from rich.syntax import Syntax, SyntaxTheme
from rich.text import Text
from rich.table import Table
from rich.rule import Rule

from pygments.lexers import guess_lexer
from pygments.util import ClassNotFound

__all__ = ("RST", "ReStructuredText", "reStructuredText", "RestructuredText")
__author__ = "Arian Mollik Wasi (aka. Wasi Master)"
__version__ = "1.3.2"


# ── Custom nodes for Sphinx directives ───────────────────────────────────────

class versionmodified(docutils.nodes.General, docutils.nodes.Body, docutils.nodes.Element):
    """Node produced by the versionadded, versionchanged, and deprecated directives."""
    pass


class seealso(docutils.nodes.Admonition, docutils.nodes.Element):
    """Node produced by the seealso directive."""
    pass


class centered_block(docutils.nodes.General, docutils.nodes.Body, docutils.nodes.Element):
    """Node for .. centered:: directive."""
    pass


class py_desc(docutils.nodes.General, docutils.nodes.Body, docutils.nodes.Element):
    """Node for Python/C/C++/JS domain object-description directives."""
    pass


class toctree_stub(docutils.nodes.General, docutils.nodes.Body, docutils.nodes.Element):
    """Node for .. toctree:: directive."""
    pass


class literalinclude_stub(docutils.nodes.General, docutils.nodes.Body, docutils.nodes.Element):
    """Node for .. literalinclude:: directive."""
    pass


# ── Docutils directive classes for Sphinx-specific directives ─────────────────

class _VersionDirective(docutils.parsers.rst.Directive):
    """Handles ``.. versionadded::``, ``.. versionchanged::``, and ``.. deprecated::``."""

    required_arguments = 1
    optional_arguments = 0
    final_argument_whitespace = True
    option_spec = {}
    has_content = True

    def run(self):
        node = versionmodified(type=self.name, version=self.arguments[0])
        if self.content:
            self.state.nested_parse(self.content, self.content_offset, node)
        return [node]


class _SeeAlsoDirective(docutils.parsers.rst.Directive):
    """Handles ``.. seealso::``."""

    required_arguments = 0
    optional_arguments = 1
    final_argument_whitespace = True
    option_spec = {}
    has_content = True

    def run(self):
        node = seealso()
        if self.arguments:
            node += docutils.nodes.paragraph(self.arguments[0], self.arguments[0])
        if self.content:
            self.state.nested_parse(self.content, self.content_offset, node)
        return [node]


class _CodeBlockDirective(docutils.parsers.rst.Directive):
    """Handles ``.. code-block::``, ``.. sourcecode::``, ``.. code::``."""

    required_arguments = 0
    optional_arguments = 1
    final_argument_whitespace = False
    has_content = True
    option_spec = {
        'linenos': docutils.parsers.rst.directives.flag,
        'emphasize-lines': docutils.parsers.rst.directives.unchanged,
        'caption': docutils.parsers.rst.directives.unchanged,
        'name': docutils.parsers.rst.directives.unchanged,
        'dedent': docutils.parsers.rst.directives.unchanged,
        'force': docutils.parsers.rst.directives.flag,
        'class': docutils.parsers.rst.directives.unchanged,
        'number-lines': docutils.parsers.rst.directives.nonnegative_int,
    }

    def run(self):
        language = self.arguments[0] if self.arguments else None
        code = '\n'.join(self.content)
        node = docutils.nodes.literal_block(code, code)
        if language:
            node['classes'] = ['code', language]
        else:
            node['classes'] = ['code']
        return [node]


class _HighlightDirective(docutils.parsers.rst.Directive):
    """Handles ``.. highlight::``."""

    required_arguments = 1
    optional_arguments = 0
    final_argument_whitespace = False
    has_content = False
    option_spec = {
        'linenothreshold': docutils.parsers.rst.directives.nonnegative_int,
        'force': docutils.parsers.rst.directives.flag,
    }

    def run(self):
        return []


class _SilentDirective(docutils.parsers.rst.Directive):
    """No-op directive for index, tabularcolumns, etc."""

    required_arguments = 0
    optional_arguments = 1
    final_argument_whitespace = True
    has_content = True
    option_spec = {}

    def run(self):
        return []


class _CurrentModuleDirective(docutils.parsers.rst.Directive):
    """No-op directive for currentmodule, py:currentmodule."""

    required_arguments = 1
    optional_arguments = 0
    final_argument_whitespace = False
    has_content = False
    option_spec = {}

    def run(self):
        return []


class _OnlyDirective(docutils.parsers.rst.Directive):
    """Handles ``.. only::`` — always renders content."""

    required_arguments = 1
    optional_arguments = 0
    final_argument_whitespace = True
    has_content = True
    option_spec = {}

    def run(self):
        container = docutils.nodes.container()
        if self.content:
            self.state.nested_parse(self.content, self.content_offset, container)
        return container.children


class _CenteredDirective(docutils.parsers.rst.Directive):
    """Handles ``.. centered::``."""

    required_arguments = 1
    optional_arguments = 0
    final_argument_whitespace = True
    has_content = False
    option_spec = {}

    def run(self):
        return [centered_block(text=self.arguments[0])]


class _HlistDirective(docutils.parsers.rst.Directive):
    """Handles ``.. hlist::`` — renders as plain bullet list."""

    required_arguments = 0
    optional_arguments = 0
    final_argument_whitespace = False
    has_content = True
    option_spec = {
        'columns': docutils.parsers.rst.directives.nonnegative_int,
    }

    def run(self):
        container = docutils.nodes.container()
        if self.content:
            self.state.nested_parse(self.content, self.content_offset, container)
        return container.children


class _ToctreeDirective(docutils.parsers.rst.Directive):
    """Handles ``.. toctree::``."""

    required_arguments = 0
    optional_arguments = 0
    final_argument_whitespace = False
    has_content = True
    option_spec = {
        'maxdepth': docutils.parsers.rst.directives.nonnegative_int,
        'caption': docutils.parsers.rst.directives.unchanged,
        'name': docutils.parsers.rst.directives.unchanged,
        'titlesonly': docutils.parsers.rst.directives.flag,
        'glob': docutils.parsers.rst.directives.flag,
        'hidden': docutils.parsers.rst.directives.flag,
        'includehidden': docutils.parsers.rst.directives.flag,
        'reversed': docutils.parsers.rst.directives.flag,
        'numbered': docutils.parsers.rst.directives.nonnegative_int,
    }

    def run(self):
        caption = self.options.get('caption', 'Contents')
        entries = [
            line.strip() for line in self.content
            if line.strip() and not line.strip().startswith(':')
        ]
        node = toctree_stub()
        node['caption'] = caption
        node['entries'] = entries
        return [node]


class _LiteralIncludeDirective(docutils.parsers.rst.Directive):
    """Handles ``.. literalinclude::``."""

    required_arguments = 1
    optional_arguments = 0
    final_argument_whitespace = False
    has_content = False
    option_spec = {
        'language': docutils.parsers.rst.directives.unchanged,
        'linenos': docutils.parsers.rst.directives.flag,
        'lines': docutils.parsers.rst.directives.unchanged,
        'start-after': docutils.parsers.rst.directives.unchanged,
        'end-before': docutils.parsers.rst.directives.unchanged,
        'encoding': docutils.parsers.rst.directives.unchanged,
        'dedent': docutils.parsers.rst.directives.unchanged,
        'tab-width': docutils.parsers.rst.directives.nonnegative_int,
        'caption': docutils.parsers.rst.directives.unchanged,
        'name': docutils.parsers.rst.directives.unchanged,
        'start-at': docutils.parsers.rst.directives.unchanged,
        'end-at': docutils.parsers.rst.directives.unchanged,
        'prepend': docutils.parsers.rst.directives.unchanged,
        'append': docutils.parsers.rst.directives.unchanged,
        'force': docutils.parsers.rst.directives.flag,
        'diff': docutils.parsers.rst.directives.unchanged,
    }

    def run(self):
        node = literalinclude_stub()
        node['filename'] = self.arguments[0]
        return [node]


class _ProductionListDirective(docutils.parsers.rst.Directive):
    """Handles ``.. productionlist::``."""

    required_arguments = 0
    optional_arguments = 1
    final_argument_whitespace = True
    has_content = True
    option_spec = {}

    def run(self):
        if self.content:
            code = '\n'.join(self.content)
        elif self.arguments:
            code = self.arguments[0]
        else:
            code = ''
        node = docutils.nodes.literal_block(code, code)
        node['classes'] = ['code', 'text']
        return [node]


class _GlossaryDirective(docutils.parsers.rst.Directive):
    """Handles ``.. glossary::``."""

    required_arguments = 0
    optional_arguments = 0
    final_argument_whitespace = False
    has_content = True
    option_spec = {
        'sorted': docutils.parsers.rst.directives.flag,
    }

    def run(self):
        container = docutils.nodes.container()
        if self.content:
            self.state.nested_parse(self.content, self.content_offset, container)
        return container.children


class _DeprecatedRemovedDirective(docutils.parsers.rst.Directive):
    """Handles ``.. deprecated-removed::``."""

    required_arguments = 2
    optional_arguments = 0
    final_argument_whitespace = False
    has_content = True
    option_spec = {}

    def run(self):
        version_str = f"{self.arguments[0]} (removed in {self.arguments[1]})"
        node = versionmodified(type='deprecated-removed', version=version_str)
        if self.content:
            self.state.nested_parse(self.content, self.content_offset, node)
        return [node]


class _PyObjectDirective(docutils.parsers.rst.Directive):
    """Handles Python/C/C++/JS domain object-description directives."""

    required_arguments = 1
    optional_arguments = 0
    final_argument_whitespace = True
    has_content = True
    option_spec = {
        'no-index': docutils.parsers.rst.directives.flag,
        'noindex': docutils.parsers.rst.directives.flag,
        'module': docutils.parsers.rst.directives.unchanged,
        'annotation': docutils.parsers.rst.directives.unchanged,
        'type': docutils.parsers.rst.directives.unchanged,
        'value': docutils.parsers.rst.directives.unchanged,
        'async': docutils.parsers.rst.directives.flag,
        'classmethod': docutils.parsers.rst.directives.flag,
        'staticmethod': docutils.parsers.rst.directives.flag,
        'abstract': docutils.parsers.rst.directives.flag,
        'final': docutils.parsers.rst.directives.flag,
        'canonical': docutils.parsers.rst.directives.unchanged,
        'platform': docutils.parsers.rst.directives.unchanged,
        'synopsis': docutils.parsers.rst.directives.unchanged,
        'deprecated': docutils.parsers.rst.directives.flag,
    }

    def run(self):
        if ':' in self.name:
            _, _, objtype = self.name.partition(':')
        else:
            objtype = self.name
        node = py_desc(objtype=objtype, sig=self.arguments[0])
        if self.content:
            self.state.nested_parse(self.content, self.content_offset, node)
        return [node]


class _AutodocDirective(docutils.parsers.rst.Directive):
    """Handles autodoc directives (automodule, autoclass, etc.)."""

    required_arguments = 1
    optional_arguments = 0
    final_argument_whitespace = True
    has_content = True
    option_spec = {
        'members': docutils.parsers.rst.directives.unchanged,
        'undoc-members': docutils.parsers.rst.directives.flag,
        'show-inheritance': docutils.parsers.rst.directives.flag,
        'member-order': docutils.parsers.rst.directives.unchanged,
        'exclude-members': docutils.parsers.rst.directives.unchanged,
        'private-members': docutils.parsers.rst.directives.unchanged,
        'special-members': docutils.parsers.rst.directives.unchanged,
        'inherited-members': docutils.parsers.rst.directives.unchanged,
        'no-index': docutils.parsers.rst.directives.flag,
        'noindex': docutils.parsers.rst.directives.flag,
        'synopsis': docutils.parsers.rst.directives.unchanged,
        'platform': docutils.parsers.rst.directives.unchanged,
        'deprecated': docutils.parsers.rst.directives.flag,
        'ignore-module-all': docutils.parsers.rst.directives.flag,
    }

    def run(self):
        return []


_sphinx_directives_registered = False


def _register_sphinx_directives():
    """Register Sphinx-specific directives so they render properly instead of as errors."""
    global _sphinx_directives_registered

    if _sphinx_directives_registered:
        return

    docutils.parsers.rst.directives.register_directive('versionadded', _VersionDirective)
    docutils.parsers.rst.directives.register_directive('versionchanged', _VersionDirective)
    docutils.parsers.rst.directives.register_directive('deprecated', _VersionDirective)
    docutils.parsers.rst.directives.register_directive('seealso', _SeeAlsoDirective)

    # code-block
    for name in ('code-block', 'sourcecode', 'code'):
        docutils.parsers.rst.directives.register_directive(name, _CodeBlockDirective)
    # highlight
    docutils.parsers.rst.directives.register_directive('highlight', _HighlightDirective)
    # silent no-op
    for name in ('index', 'tabularcolumns'):
        docutils.parsers.rst.directives.register_directive(name, _SilentDirective)
    # current module
    for name in ('currentmodule', 'py:currentmodule'):
        docutils.parsers.rst.directives.register_directive(name, _CurrentModuleDirective)
    # only
    docutils.parsers.rst.directives.register_directive('only', _OnlyDirective)
    # centered
    docutils.parsers.rst.directives.register_directive('centered', _CenteredDirective)
    # hlist
    docutils.parsers.rst.directives.register_directive('hlist', _HlistDirective)
    # toctree
    docutils.parsers.rst.directives.register_directive('toctree', _ToctreeDirective)
    # literalinclude
    docutils.parsers.rst.directives.register_directive('literalinclude', _LiteralIncludeDirective)
    # productionlist
    docutils.parsers.rst.directives.register_directive('productionlist', _ProductionListDirective)
    # glossary
    docutils.parsers.rst.directives.register_directive('glossary', _GlossaryDirective)
    # deprecated-removed
    docutils.parsers.rst.directives.register_directive('deprecated-removed', _DeprecatedRemovedDirective)
    # Python domain object descriptions
    for name in (
        'py:function', 'py:class', 'py:method', 'py:attribute', 'py:data',
        'py:exception', 'py:module', 'py:property', 'py:decorator',
        'py:classmethod', 'py:staticmethod', 'py:variable', 'py:type',
        'py:typevar', 'py:typealias',
        # C domain
        'c:function', 'c:type', 'c:struct', 'c:union', 'c:enum',
        'c:enumerator', 'c:member', 'c:var', 'c:macro',
        # C++ domain
        'cpp:function', 'cpp:class', 'cpp:type', 'cpp:member', 'cpp:var',
        'cpp:enum', 'cpp:enumerator', 'cpp:concept', 'cpp:alias',
        # JavaScript domain
        'js:function', 'js:class', 'js:method', 'js:attribute', 'js:data',
        'js:module',
    ):
        docutils.parsers.rst.directives.register_directive(name, _PyObjectDirective)
    # autodoc
    for name in (
        'automodule', 'autoclass', 'autofunction', 'automethod',
        'autoattribute', 'autoexception', 'autodata', 'autoproperty',
        'autodecorator', 'autoclassmethod', 'autostaticmethod',
    ):
        docutils.parsers.rst.directives.register_directive(name, _AutodocDirective)

    _sphinx_directives_registered = True


_sphinx_roles_registered = False



class MLStripper(HTMLParser):
    """Utility class to strip out html for raw html source"""
    def __init__(self):
        super().__init__()
        self.reset()
        self.convert_charrefs = True
        self.text = StringIO()

    def handle_data(self, d):
        self.text.write(d)

    def get_data(self):
        return self.text.getvalue()


def strip_tags(html):
    s = MLStripper()
    s.feed(html)
    return s.get_data()


def _register_sphinx_roles():
    """
    Register common Sphinx roles to gracefully handle Sphinx-specific markup.

    Sphinx roles like :func:, :class:, :meth: are very common in Python docstrings
    but are not available in standard docutils. This function registers them to
    render as inline code/literal text instead of showing errors.
    """
    global _sphinx_roles_registered

    if _sphinx_roles_registered:
        return

    from rich_rst._vendor import docutils
    import rich_rst._vendor.docutils.parsers.rst.roles
    import rich_rst._vendor.docutils.parsers.rst.languages.en

    def sphinx_role(name, rawtext, text, lineno, inliner, options={}, content=[]):
        """
        Generic Sphinx role handler that renders as inline literal text.

        Parameters
        ----------
        name : str
            The role name
        rawtext : str
            The entire role text including role markup
        text : str
            The interpreted text content
        lineno : int
            The line number where the interpreted text begins
        inliner : Inliner
            The inliner instance that called this role function
        options : dict
            Directive options for customization
        content : list
            The directive content for customization

        Returns
        -------
        tuple
            A tuple of (nodes, messages)
        """
        display_text = text
        if '<' in text and text.endswith('>'):
            bracket_pos = text.rfind('<')
            potential_display = text[:bracket_pos].strip()
            if potential_display:
                display_text = potential_display

        node = docutils.nodes.literal(rawtext, display_text)
        return [node], []

    sphinx_roles = [
        'func', 'function',
        'meth', 'method',
        'class',
        'mod', 'module',
        'attr', 'attribute',
        'obj', 'object',
        'data',
        'const', 'constant',
        'exc', 'exception',
        'var', 'variable',
        'type',
        'py:func', 'py:meth', 'py:class', 'py:mod', 'py:attr',
        'py:obj', 'py:data', 'py:const', 'py:exc',
        # Standard domain cross-reference roles
        'envvar', 'token', 'option', 'term', 'ref', 'doc', 'any', 'numref', 'download',
        # Misc
        'mailheader', 'mimetype', 'newsgroup', 'makevar', 'regexp',
        # Keyboard/GUI
        'kbd', 'guilabel',
        # Unix man pages
        'manpage',
        # Python domain additional
        'py:variable', 'py:type', 'py:property', 'py:parameter', 'py:typevar',
        # C domain
        'c:func', 'c:type', 'c:struct', 'c:union', 'c:enum', 'c:enumerator',
        'c:member', 'c:var', 'c:macro', 'c:expr', 'c:texpr',
        # C++ domain
        'cpp:func', 'cpp:class', 'cpp:type', 'cpp:member', 'cpp:var',
        'cpp:enum', 'cpp:enumerator', 'cpp:concept', 'cpp:expr', 'cpp:texpr',
        # JavaScript domain
        'js:mod', 'js:func', 'js:data', 'js:attr', 'js:class', 'js:meth',
    ]

    for role in sphinx_roles:
        docutils.parsers.rst.roles.register_canonical_role(role, sphinx_role)
        # Also register in language module to avoid INFO messages
        if hasattr(docutils.parsers.rst.languages.en, 'roles'):
            docutils.parsers.rst.languages.en.roles[role] = role

    import re as _re

    # `:command:` and `:program:` → bold literal
    def _bold_literal_role(name, rawtext, text, lineno, inliner, options={}, content=[]):
        display_text = text
        if '<' in text and text.endswith('>'):
            bracket_pos = text.rfind('<')
            potential_display = text[:bracket_pos].strip()
            if potential_display:
                display_text = potential_display
        node = docutils.nodes.strong(rawtext, display_text)
        return [node], []

    for _role_name in ('command', 'program'):
        docutils.parsers.rst.roles.register_canonical_role(_role_name, _bold_literal_role)
        if hasattr(docutils.parsers.rst.languages.en, 'roles'):
            docutils.parsers.rst.languages.en.roles[_role_name] = _role_name

    # `:dfn:` → emphasis (italic)
    def _dfn_role(name, rawtext, text, lineno, inliner, options={}, content=[]):
        node = docutils.nodes.emphasis(rawtext, text)
        return [node], []

    docutils.parsers.rst.roles.register_canonical_role('dfn', _dfn_role)
    if hasattr(docutils.parsers.rst.languages.en, 'roles'):
        docutils.parsers.rst.languages.en.roles['dfn'] = 'dfn'

    # `:abbr:` → abbreviation node with explanation
    _abbr_re = _re.compile(r'\((.*)\)$', _re.DOTALL)

    def _abbr_role(name, rawtext, text, lineno, inliner, options={}, content=[]):
        matched = _abbr_re.search(text)
        if matched:
            abbr_text = text[:matched.start()].strip()
            explanation = matched.group(1)
        else:
            abbr_text = text
            explanation = ''
        node = docutils.nodes.abbreviation(rawtext, abbr_text, explanation=explanation)
        return [node], []

    docutils.parsers.rst.roles.register_canonical_role('abbr', _abbr_role)
    if hasattr(docutils.parsers.rst.languages.en, 'roles'):
        docutils.parsers.rst.languages.en.roles['abbr'] = 'abbr'

    # `:menuselection:` → replace `-->` with ` ▶ `
    def _menuselection_role(name, rawtext, text, lineno, inliner, options={}, content=[]):
        text = text.replace('-->', '\u25b6')
        node = docutils.nodes.literal(rawtext, text)
        return [node], []

    docutils.parsers.rst.roles.register_canonical_role('menuselection', _menuselection_role)
    if hasattr(docutils.parsers.rst.languages.en, 'roles'):
        docutils.parsers.rst.languages.en.roles['menuselection'] = 'menuselection'

    # `:samp:` and `:file:` → literal with {} stripped
    _braces_re = _re.compile(r'\{([^}]*)\}')

    def _samp_role(name, rawtext, text, lineno, inliner, options={}, content=[]):
        clean = _braces_re.sub(r'\1', text)
        node = docutils.nodes.literal(rawtext, clean)
        return [node], []

    for _role_name in ('samp', 'file'):
        docutils.parsers.rst.roles.register_canonical_role(_role_name, _samp_role)
        if hasattr(docutils.parsers.rst.languages.en, 'roles'):
            docutils.parsers.rst.languages.en.roles[_role_name] = _role_name

    # `:pep:` → bold text with clickable PEP link
    def _pep_role(name, rawtext, text, lineno, inliner, options={}, content=[]):
        parts = text.split('#', 1)
        pep_num_str = parts[0].strip()
        anchor = ('#' + parts[1]) if len(parts) > 1 else ''
        try:
            pep_num = int(pep_num_str)
            url = f"https://peps.python.org/pep-{pep_num:04d}/{anchor}"
        except ValueError:
            url = "https://peps.python.org/"
        display = f"PEP {pep_num_str} <{url}>"
        ref = docutils.nodes.reference(rawtext, display, refuri=url)
        return [ref], []

    docutils.parsers.rst.roles.register_canonical_role('pep', _pep_role)
    if hasattr(docutils.parsers.rst.languages.en, 'roles'):
        docutils.parsers.rst.languages.en.roles['pep'] = 'pep'

    # `:rfc:` → bold text with clickable RFC link
    def _rfc_role(name, rawtext, text, lineno, inliner, options={}, content=[]):
        parts = text.split('#', 1)
        rfc_num_str = parts[0].strip()
        anchor = ('#' + parts[1]) if len(parts) > 1 else ''
        try:
            rfc_num = int(rfc_num_str)
            url = f"https://datatracker.ietf.org/doc/html/rfc{rfc_num}{anchor}"
        except ValueError:
            url = "https://datatracker.ietf.org/"
        display = f"RFC {rfc_num_str} <{url}>"
        ref = docutils.nodes.reference(rawtext, display, refuri=url)
        return [ref], []

    docutils.parsers.rst.roles.register_canonical_role('rfc', _rfc_role)
    if hasattr(docutils.parsers.rst.languages.en, 'roles'):
        docutils.parsers.rst.languages.en.roles['rfc'] = 'rfc'

    _sphinx_roles_registered = True


# pylama:ignore=D,C0116
class RSTVisitor(docutils.nodes.SparseNodeVisitor):
    """A visitor that produces rich renderables"""

    def __init__(
        self,
        document: docutils.nodes.document,
        console: Console,
        code_theme: Union[str, SyntaxTheme] = "monokai",
        show_line_numbers: Optional[bool] = False,
        guess_lexer: Optional[bool] = True,
        default_lexer: Optional[str] = "python",
    ) -> None:
        super().__init__(document)
        self.console = console
        self.code_theme = code_theme
        self.show_line_numbers = show_line_numbers
        self.renderables = []
        self.superscript = str.maketrans(
            "1234567890abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ=+-*/×÷",
            "¹²³⁴⁵⁶⁷⁸⁹⁰ᵃᵇᶜᵈᵉᶠᵍʰⁱʲᵏˡᵐⁿᵒᵖᑫʳˢᵗᵘᵛʷˣʸᶻᴬᴮᶜᴰᴱᶠᴳᴴᴵᴶᴷᴸᴹᴺᴼᴾQᴿˢᵀᵁⱽᵂˣʸᶻ⁼⁺⁻*/×÷",
        )
        self.subscript = str.maketrans(
            "1234567890abcdefghijklmnopqrstuvwxyz=+-*/×÷", "₁₂₃₄₅₆₇₈₉₀abcdₑfgₕᵢⱼₖₗₘₙₒₚqᵣₛₜᵤᵥwₓyz₌₊₋*/×÷"
        )
        self.errors = []
        self.footer = []
        self.guess_lexer = guess_lexer
        self.default_lexer = default_lexer
        self.refname_to_renderable = {}

    def _guess_lexer_name(self, text):
        try:
            lexer = guess_lexer(text)
        except ClassNotFound:
            return self.default_lexer
        return lexer.aliases[0] if lexer.aliases else self.default_lexer

    def _find_lexer(self, node):
        lexer = (
            node["classes"][1] if len(node.get("classes")) >= 2 else (node["format"] if node.get("format") else None)
        )
        if lexer is None and self.guess_lexer:
            lexer = self._guess_lexer_name(node.astext())
            if lexer == "text":
                return self.default_lexer
            return lexer
        elif lexer is None and not self.guess_lexer:
            lexer = self.default_lexer
            return lexer
        return lexer

    def _section_level(self, node):
        level = -1
        parent = getattr(node, "parent", None)
        while parent is not None:
            if isinstance(parent, docutils.nodes.section):
                level += 1
            parent = getattr(parent, "parent", None)
        return level

    def _render_heading(self, text, level):
        heading_levels = [
            ("restructuredtext.title.level.1", "bold", box.DOUBLE),
            ("restructuredtext.title.level.2", "bold", box.ROUNDED),
            ("restructuredtext.title.level.3", "bold underline", None),
            ("restructuredtext.title.level.4", "bold", None),
            ("restructuredtext.title.level.5", "underline", None),
            ("restructuredtext.title.level.6", "italic", None),
        ]
        index = min(level, len(heading_levels) - 1)
        style_name, default_style, panel_box = heading_levels[index]
        style = self.console.get_style(style_name, default=default_style)
        if panel_box is None:
            self.renderables.append(Align(Text(text, style=style), "center"))
            self.renderables.append(NewLine())
        else:
            self.renderables.append(Panel(Align(text, "center"), box=panel_box, style=style, border_style=style))

    def visit_reference(self, node):
        if len(node.children) == 1 and isinstance(node.children[0], docutils.nodes.image):
            return
        refuri = node.attributes.get("refuri")
        style = self.console.get_style("restructuredtext.reference", default="blue underline on default")
        if refuri:
            style = style.update_link(refuri)
        renderable = Text(node.astext().replace("\n", " "), style=style, end="")
        if self.renderables and isinstance(self.renderables[-1], Text):
            renderable.end = " "
            start = len(self.renderables[-1])
            self.renderables[-1].append_text(renderable)
        else:
            start = 0
            self.renderables.append(renderable)
        end = len(self.renderables[-1])

        if not refuri:
            # We'll get the URL reference later in visit_target.
            refname = node.attributes.get("refname")
            if refname:
                self.refname_to_renderable[refname] = (self.renderables[-1], start, end)
        raise docutils.nodes.SkipChildren()

    def visit_target(self, node):
        uri = node.get("refuri")
        if uri:
            for name in node["names"]:
                try:
                    renderable, start, end = self.refname_to_renderable[name]
                except KeyError:
                    continue
                style = renderable.get_style_at_offset(self.console, start)
                style = style.update_link(uri)
                renderable.stylize(style, start, end)
        raise docutils.nodes.SkipChildren()

    def visit_paragraph(self, node):
        if hasattr(node, "parent") and isinstance(node.parent, docutils.nodes.system_message):
            self.visit_system_message(node.parent)
            raise docutils.nodes.SkipChildren()

    def depart_paragraph(self, node):  # pylint: disable=unused-argument
        if self.renderables and isinstance(self.renderables[-1], Text):
            if len(self.renderables[-1].end) == 0:
                self.renderables[-1].append("\n\n")

    def visit_title(self, node):
        level = self._section_level(node)
        self._render_heading(node.astext(), level)
        raise docutils.nodes.SkipChildren()

    def visit_rubric(self, node):
        style = self.console.get_style("restructuredtext.rubric", default="italic dim")
        self.renderables.append(Panel(Align(node.astext(), "center"), box=box.ROUNDED, style=style, border_style=style))
        raise docutils.nodes.SkipChildren()

    def visit_Text(self, node):
        style = self.console.get_style("restructuredtext.text", default="default on default not underline")
        if self.renderables and isinstance(self.renderables[-1], Text):
            self.renderables[-1].append_text(Text(node.astext().replace("\n", " "), style=style, end=" "))
            return
        self.renderables.append(Text(node.astext().replace("\n", " "), end="", style=style))

    def visit_comment(self, node):
        raise docutils.nodes.SkipChildren()

    def visit_substitution_definition(self, node):
        raise docutils.nodes.SkipChildren()

    def _render_admonition_body(self, children):
        """Render admonition body children using a sub-visitor to preserve inline markup."""
        sub_visitor = RSTVisitor(
            self.document,
            console=self.console,
            code_theme=self.code_theme,
            show_line_numbers=self.show_line_numbers,
            guess_lexer=self.guess_lexer,
            default_lexer=self.default_lexer,
        )
        for child in children:
            child.walkabout(sub_visitor)
        return sub_visitor.renderables

    def visit_admonition(self, node):
        style = self.console.get_style("restructuredtext.admonition", default="bold white")
        # Generic admonition: first child is the user-supplied title node
        if node.children and isinstance(node.children[0], docutils.nodes.title):
            title = "Admonition: " + node.children[0].astext()
            body_children = node.children[1:]
        else:
            title = "Admonition: "
            body_children = node.children
        body = self._render_admonition_body(body_children)
        self.renderables.append(Panel(Group(*body) if body else "", title=title, style=style, border_style=style))
        raise docutils.nodes.SkipChildren()

    def visit_attention(self, node):
        style = self.console.get_style("restructuredtext.attention", default="bold black on yellow")
        body = self._render_admonition_body(node.children)
        self.renderables.append(Panel(Group(*body) if body else "", title="Attention: ", style=style, border_style=style))
        raise docutils.nodes.SkipChildren()

    def visit_caution(self, node):
        style = self.console.get_style("restructuredtext.caution", default="red")
        body = self._render_admonition_body(node.children)
        self.renderables.append(Panel(Group(*body) if body else "", title="Caution: ", style=style, border_style=style))
        raise docutils.nodes.SkipChildren()

    def visit_danger(self, node):
        style = self.console.get_style("restructuredtext.danger", default="bold white on red")
        body = self._render_admonition_body(node.children)
        self.renderables.append(Panel(Group(*body) if body else "", title="DANGER: ", style=style, border_style=style))
        raise docutils.nodes.SkipChildren()

    def visit_error(self, node):
        style = self.console.get_style("restructuredtext.error", default="bold red")
        body = self._render_admonition_body(node.children)
        self.renderables.append(Panel(Group(*body) if body else "", title="ERROR: ", style=style, border_style=style))
        raise docutils.nodes.SkipChildren()

    def visit_hint(self, node):
        style = self.console.get_style("restructuredtext.hint", default="yellow")
        body = self._render_admonition_body(node.children)
        self.renderables.append(Panel(Group(*body) if body else "", title="Hint: ", style=style, border_style=style))
        raise docutils.nodes.SkipChildren()

    def visit_important(self, node):
        style = self.console.get_style("restructuredtext.important", default="bold blue")
        body = self._render_admonition_body(node.children)
        self.renderables.append(Panel(Group(*body) if body else "", title="IMPORTANT: ", style=style, border_style=style))
        raise docutils.nodes.SkipChildren()

    def visit_note(self, node):
        style = self.console.get_style("restructuredtext.note", default="bold white")
        body = self._render_admonition_body(node.children)
        self.renderables.append(Panel(Group(*body) if body else "", title="Note: ", style=style, border_style=style))
        raise docutils.nodes.SkipChildren()

    def visit_tip(self, node):
        style = self.console.get_style("restructuredtext.tip", default="bold green")
        body = self._render_admonition_body(node.children)
        self.renderables.append(Panel(Group(*body) if body else "", title="Tip: ", style=style, border_style=style))
        raise docutils.nodes.SkipChildren()

    def visit_warning(self, node):
        style = self.console.get_style("restructuredtext.warning", default="bold yellow")
        body = self._render_admonition_body(node.children)
        self.renderables.append(Panel(Group(*body) if body else "", title="Warning: ", style=style, border_style=style))
        raise docutils.nodes.SkipChildren()

    def visit_versionmodified(self, node):
        type_ = node.get("type", "versionadded")
        version = node.get("version", "")
        style_map = {
            "versionadded": ("restructuredtext.versionadded", "bold green"),
            "versionchanged": ("restructuredtext.versionchanged", "bold cyan"),
            "deprecated": ("restructuredtext.deprecated", "bold yellow"),
            "deprecated-removed": ("restructuredtext.deprecated_removed", "bold red"),
        }
        title_map = {
            "versionadded": f"New in version {version}",
            "versionchanged": f"Changed in version {version}",
            "deprecated": f"Deprecated since version {version}",
            "deprecated-removed": f"Deprecated since version {version}",
        }
        style_name, default_style = style_map.get(type_, ("restructuredtext.versionadded", "bold green"))
        title = title_map.get(type_, f"{type_} {version}")
        style = self.console.get_style(style_name, default=default_style)
        body = self._render_admonition_body(node.children)
        self.renderables.append(Panel(Group(*body) if body else "", title=title, style=style, border_style=style))
        raise docutils.nodes.SkipChildren()

    def depart_versionmodified(self, node):
        pass

    def visit_seealso(self, node):
        style = self.console.get_style("restructuredtext.seealso", default="bold white")
        body = self._render_admonition_body(node.children)
        self.renderables.append(Panel(Group(*body) if body else "", title="See Also", style=style, border_style=style))
        raise docutils.nodes.SkipChildren()

    def depart_seealso(self, node):
        pass

    def visit_centered_block(self, node):
        style = self.console.get_style("restructuredtext.centered", default="bold")
        text = node.get('text', '')
        self.renderables.append(Align(Text(text, style=style), "center"))
        raise docutils.nodes.SkipChildren()

    def depart_centered_block(self, node):
        pass

    def visit_py_desc(self, node):
        objtype = node.get('objtype', 'object')
        sig = node.get('sig', '')
        style = self.console.get_style("restructuredtext.py_desc", default="bold blue")
        body = self._render_admonition_body(node.children)
        self.renderables.append(
            Panel(Group(*body) if body else "", title=f"[{objtype}] {sig}",
                  style=style, border_style=style)
        )
        raise docutils.nodes.SkipChildren()

    def depart_py_desc(self, node):
        pass

    def visit_toctree_stub(self, node):
        style = self.console.get_style("restructuredtext.toctree", default="bold cyan")
        caption = node.get('caption', 'Contents')
        entries = node.get('entries', [])
        marker_style = self.console.get_style("restructuredtext.bullet_list_marker", default="bold yellow")
        renderables = [Text(" • " + e, style=marker_style) for e in entries if e]
        self.renderables.append(
            Panel(Group(*renderables) if renderables else "", title=caption,
                  style=style, border_style=style)
        )
        raise docutils.nodes.SkipChildren()

    def depart_toctree_stub(self, node):
        pass

    def visit_literalinclude_stub(self, node):
        style = self.console.get_style("restructuredtext.literalinclude", default="grey58")
        filename = node.get('filename', '<unknown file>')
        self.renderables.append(
            Panel(Text(filename), title="literalinclude", border_style=style)
        )
        raise docutils.nodes.SkipChildren()

    def depart_literalinclude_stub(self, node):
        pass

    def visit_subscript(self, node):
        style = self.console.get_style("restructuredtext.subscript", default="none")
        if self.renderables and isinstance(self.renderables[-1], Text):
            self.renderables[-1].append_text(Text(node.astext().translate(self.subscript), style=style, end=" "))
            raise docutils.nodes.SkipChildren()
        self.renderables.append(Text(node.astext().translate(self.subscript), end="", style=style))
        raise docutils.nodes.SkipChildren()

    def visit_superscript(self, node):
        style = self.console.get_style("restructuredtext.superscript", default="none")
        if self.renderables and isinstance(self.renderables[-1], Text):
            self.renderables[-1].append_text(Text(node.astext().translate(self.superscript), style=style, end=" "))
            raise docutils.nodes.SkipChildren()
        self.renderables.append(Text(node.astext().translate(self.superscript), end="", style=style))
        raise docutils.nodes.SkipChildren()

    def visit_emphasis(self, node):
        style = self.console.get_style("restructuredtext.emphasis", default="italic")
        if self.renderables and isinstance(self.renderables[-1], Text):
            self.renderables[-1].append_text(Text(node.astext().replace("\n", " "), style=style, end=" "))
            raise docutils.nodes.SkipChildren()
        self.renderables.append(Text(node.astext().replace("\n", " "), style=style, end=""))
        raise docutils.nodes.SkipChildren()

    def visit_strong(self, node):
        style = self.console.get_style("restructuredtext.strong", default="bold")
        if self.renderables and isinstance(self.renderables[-1], Text):
            self.renderables[-1].append_text(Text(node.astext().replace("\n", " "), style=style, end=" "))
            raise docutils.nodes.SkipChildren()
        self.renderables.append(Text(node.astext().replace("\n", " "), style=style, end=""))
        raise docutils.nodes.SkipChildren()

    def _make_image_text(self, node, link_override=None):
        alt, target = None, None
        if ":target:" in node.rawsource:
            target = node.rawsource.split(":target:")[-1].strip()
        if ":alt:" in node.rawsource:
            alt = node.rawsource.split(":alt:")[-1].strip()
        link = link_override or node.get("target", target or "Image") or node.get("uri")
        return Text("🌆 ") + Text(
            node.get("alt", alt or "Image"),
            style=Style(link=link, color="#6088ff"),
        )


    def _render_inline_with_explanation(self, node, style_name):
        style = self.console.get_style(style_name, default="underline")
        explanation = node.get("explanation", "")
        text = node.astext().replace("\n", " ")
        if explanation:
            text = f"{text} ({explanation})"
        if self.renderables and isinstance(self.renderables[-1], Text):
            self.renderables[-1].append_text(Text(text, style=style, end=" "))
            raise docutils.nodes.SkipChildren()
        self.renderables.append(Text(text, style=style, end=""))
        raise docutils.nodes.SkipChildren()


    def visit_abbreviation(self, node):
        self._render_inline_with_explanation(node, "restructuredtext.abbreviation")


    def visit_acronym(self, node):
        self._render_inline_with_explanation(node, "restructuredtext.acronym")


    def visit_image(self, node):
        self.renderables.append(self._make_image_text(node))
        raise docutils.nodes.SkipChildren()

    def visit_figure(self, node):
        # When :target: is given, docutils wraps the image in a reference node
        ref_node = next((c for c in node.children if isinstance(c, docutils.nodes.reference)), None)
        image_node = next((c for c in node.children if isinstance(c, docutils.nodes.image)), None)
        if image_node is None and ref_node is not None:
            image_node = next((c for c in ref_node.children if isinstance(c, docutils.nodes.image)), None)
        caption_node = next((c for c in node.children if isinstance(c, docutils.nodes.caption)), None)
        legend_node = next((c for c in node.children if isinstance(c, docutils.nodes.legend)), None)

        if image_node is not None:
            link_override = ref_node.get("refuri") if ref_node is not None else None
            image_text = self._make_image_text(image_node, link_override=link_override)
        else:
            image_text = Text("🌆 Image")
        caption = caption_node.astext() if caption_node is not None else None
        subtitle = legend_node.astext().replace("\n", " ") if legend_node is not None else None

        border_style = self.console.get_style("restructuredtext.figure_border", default="blue")
        self.renderables.append(Panel(image_text, title=caption, subtitle=subtitle, border_style=border_style, expand=False))
        raise docutils.nodes.SkipChildren()

    _BULLET_LIST_MARKERS = [" • ", " ∘ ", " ▪ "]

    def _render_bullet_list(self, node, level=0):
        """Recursively render a bullet list with support for unlimited nesting and any child elements."""
        marker_style = self.console.get_style("restructuredtext.bullet_list_marker", default="bold yellow")
        text_style = self.console.get_style("restructuredtext.bullet_list_text", default="none")
        indent = "  " * level
        marker = self._BULLET_LIST_MARKERS[min(level, len(self._BULLET_LIST_MARKERS) - 1)]
        for list_item in node.children:
            first_content = True
            for child in list_item.children:
                if isinstance(child, docutils.nodes.bullet_list):
                    self._render_bullet_list(child, level + 1)
                elif isinstance(child, docutils.nodes.enumerated_list):
                    self._render_enumerated_list(child, level + 1)
                elif isinstance(child, docutils.nodes.literal_block):
                    if first_content:
                        self.renderables.append(Text(indent + marker, end="", style=marker_style))
                        first_content = False
                    try:
                        self.visit_literal_block(child)
                    except docutils.nodes.SkipChildren:
                        pass
                else:
                    text = child.astext().replace("\n", " ")
                    if first_content:
                        self.renderables.append(Text(indent + marker, end="", style=marker_style))
                        self.renderables.append(Text(text, style=text_style))
                        first_content = False
                    else:
                        self.renderables.append(Text(indent + "  " + text, style=text_style))

    def visit_bullet_list(self, node):
        self._render_bullet_list(node, level=0)
        self.renderables.append(NewLine())
        raise docutils.nodes.SkipChildren()

    def _render_enumerated_list(self, node, level=0):
        """Recursively render an enumerated list with support for unlimited nesting and any child elements."""
        marker_style = self.console.get_style("restructuredtext.enumerated_list_marker", default="bold yellow")
        text_style = self.console.get_style("restructuredtext.enumerated_text", default="none")
        indent = "  " * level
        for i, list_item in enumerate(node.children, 1):
            first_content = True
            for child in list_item.children:
                if isinstance(child, docutils.nodes.bullet_list):
                    self._render_bullet_list(child, level + 1)
                elif isinstance(child, docutils.nodes.enumerated_list):
                    self._render_enumerated_list(child, level + 1)
                elif isinstance(child, docutils.nodes.literal_block):
                    if first_content:
                        self.renderables.append(Text(f"{indent} {i}", end=" ", style=marker_style))
                        first_content = False
                    try:
                        self.visit_literal_block(child)
                    except docutils.nodes.SkipChildren:
                        pass
                else:
                    text = child.astext().replace("\n", " ")
                    if first_content:
                        self.renderables.append(Text(f"{indent} {i}", end=" ", style=marker_style))
                        self.renderables.append(Text(text, style=text_style))
                        first_content = False
                    else:
                        self.renderables.append(Text(indent + "  " + text, style=text_style))

    def visit_enumerated_list(self, node):
        self._render_enumerated_list(node, level=0)
        self.renderables.append(NewLine())
        raise docutils.nodes.SkipChildren()

    def visit_literal(self, node):
        style = self.console.get_style("restructuredtext.inline_codeblock", default="grey78 on grey7")
        if self.renderables and isinstance(self.renderables[-1], Text):
            self.renderables[-1].append_text(Text(node.astext().replace("\n", " "), style=style, end=" "))
            raise docutils.nodes.SkipChildren()
        self.renderables.append(Text(node.astext().replace("\n", " "), style=style, end=""))
        raise docutils.nodes.SkipChildren()

    def visit_title_reference(self, node):
        style = self.console.get_style("restructuredtext.title_reference", default="italic")
        if self.renderables and isinstance(self.renderables[-1], Text):
            self.renderables[-1].append_text(Text(node.astext().replace("\n", " "), style=style, end=" "))
            raise docutils.nodes.SkipChildren()
        self.renderables.append(Text(node.astext().replace("\n", " "), style=style, end=""))
        raise docutils.nodes.SkipChildren()

    def visit_literal_block(self, node):
        style = self.console.get_style("restructuredtext.literal_block_border", default="grey58")
        if self.renderables and isinstance(self.renderables[-1], Text):
            self.renderables[-1].rstrip()
            self.renderables[-1].append_text(Text("\n"))
        lexer = self._find_lexer(node)
        self.renderables.append(
            Panel(
                Syntax(node.astext(), lexer, theme=self.code_theme, line_numbers=self.show_line_numbers),
                border_style=style,
                box=box.SQUARE,
                title=lexer,
            )
        )
        raise docutils.nodes.SkipChildren()

    def visit_system_message(self, node):
        self.errors.append(
            Panel(
                Text(node.astext()),
                title=f"System Message: {node.attributes.get('type', '?')}/{node.attributes.get('level', '?')} ({node.attributes.get('source', '?')}, line {node.attributes.get('line', '?')});",
                border_style={None: "none", "INFO": "bold cyan", "WARNING": "bold yellow", "ERROR": "bold red", "SEVERE": "bold magenta", "DEBUG": "bold white"}.get(
                    node.attributes.get("type"), "bold red"
                )
            ),
        )
        raise docutils.nodes.SkipChildren()

    def _add_to_field_table(self, field_name, field_value):
        """Add a row to the shared field table, creating it if necessary."""
        field_name_style = self.console.get_style("restructuredtext.field_name", default="bold")
        field_value_style = self.console.get_style("restructuredtext.field_value", default="none")
        if self.renderables and isinstance(self.renderables[-1], Table):
            possible_table = self.renderables[-1]
            if (possible_table.columns[0].header == "Field Name") and (possible_table.columns[1].header == "Field Value"):
                possible_table.add_row(Text(field_name, style=field_name_style), Text(field_value, style=field_value_style))
                return
        table = Table("Field Name", "Field Value", show_lines=True)
        table.add_row(Text(field_name, style=field_name_style), Text(field_value, style=field_value_style))
        self.renderables.append(table)

    def visit_field(self, node):
        self._add_to_field_table(node.children[0].astext(), node.children[1].astext())
        raise docutils.nodes.SkipChildren()

    def visit_docinfo(self, node):
        pass  # let the visitor descend into child docinfo nodes

    def visit_author(self, node):
        self._add_to_field_table("Author", node.astext())
        raise docutils.nodes.SkipChildren()

    def visit_authors(self, node):
        for author in node.children:
            self._add_to_field_table("Author", author.astext())
        raise docutils.nodes.SkipChildren()

    def visit_organization(self, node):
        self._add_to_field_table("Organization", node.astext())
        raise docutils.nodes.SkipChildren()

    def visit_address(self, node):
        self._add_to_field_table("Address", node.astext())
        raise docutils.nodes.SkipChildren()

    def visit_contact(self, node):
        self._add_to_field_table("Contact", node.astext())
        raise docutils.nodes.SkipChildren()

    def visit_version(self, node):
        self._add_to_field_table("Version", node.astext())
        raise docutils.nodes.SkipChildren()

    def visit_revision(self, node):
        self._add_to_field_table("Revision", node.astext())
        raise docutils.nodes.SkipChildren()

    def visit_status(self, node):
        self._add_to_field_table("Status", node.astext())
        raise docutils.nodes.SkipChildren()

    def visit_date(self, node):
        self._add_to_field_table("Date", node.astext())
        raise docutils.nodes.SkipChildren()

    def visit_copyright(self, node):
        self._add_to_field_table("Copyright", node.astext())
        raise docutils.nodes.SkipChildren()

    def visit_definition_list(self, node):
        term_style = self.console.get_style("restructuredtext.term_style", default="none")
        classifier_style = self.console.get_style("restructuredtext.classifier_style", default="cyan")
        definitions_style = self.console.get_style("restructuredtext.definitions_style", default="none")
        for child in node.children:
            try:
                term, classifier, definitions = child.children
            except ValueError:
                term, classifier = child.children[0], child.children[1]
                if len(child.children) > 2:
                    for children in child.children[2:]:
                        if isinstance(children, docutils.nodes.bullet_list):
                            self.visit_bullet_list(children)
                        elif isinstance(children, docutils.nodes.literal_block):
                            self.visit_literal_block(children)
                        elif isinstance(children, docutils.nodes.literal):
                            self.visit_literal(children)
                        elif isinstance(children, docutils.nodes.block_quote):
                            self.visit_block_quote(children)
                else:

                    self.renderables.append(
                        Text(term.astext(), style=classifier_style)
                        + Text("\n    ", end="")
                        + Text(classifier.astext().replace("\n", " "), style=definitions_style)
                        + Text("\n      ", end="")
                    )
            else:
                self.renderables.append(
                    Text("    ")
                    + Text(term.astext(), style=term_style, end="")
                    + Text(" : ", end="")
                    + Text(classifier.astext(), style=classifier_style)
                    + Text("\n      ", end="")
                    + Text(definitions.astext().replace("\n", " "), style=definitions_style)
                    + Text("\n", end="")
                )
        raise docutils.nodes.SkipChildren()

    def visit_option_list(self, node):
        option_string_style = self.console.get_style("restructuredtext.option_string", default="none")
        option_argument_style = self.console.get_style("restructuredtext.option_argument", default="none")
        option_child_text_separator_style = self.console.get_style(
            "restructuredtext.option_child_text_separator", default="none"
        )
        option_description_style = self.console.get_style("restructuredtext.option_description", default="none")
        for option_list_item in node.children:
            option_group, description = option_list_item.children
            # option_group.child_text_separator.join(map(lambda x: x.astext(), option_group.children)))
            option_text = Text(end="")
            for option in option_group.children:
                try:
                    option_string, option_argument = option.children
                except ValueError:
                    option_string, option_argument = option.children[0], None
                option_text += (
                    Text(option_string.astext(), style=option_string_style)
                    + (Text(option_argument.astext(), style=option_argument_style) if option_argument else Text())
                    + (
                        Text(option_group.child_text_separator, style=option_child_text_separator_style)
                        if len(option_group.children) > 1
                        else Text()
                    )
                )
            if description:
                option_text += Text("\n    ")
                option_text += Text(description.astext(), style=option_description_style)
            self.renderables.append(option_text + Text("\n"))
        raise docutils.nodes.SkipChildren()

    def visit_doctest_block(self, node):
        style = self.console.get_style("restructuredtext.literal_block_border", default="grey58")
        self.renderables.append(
            Panel(
                Syntax(node.astext(), "pycon", theme=self.code_theme, line_numbers=self.show_line_numbers),
                border_style=style,
                box=box.SQUARE,
                title="doctest block",
            )
        )
        raise docutils.nodes.SkipChildren()

    def visit_block_quote(self, node):
        text_style = self.console.get_style("restructuredtext.blockquote_text", default="white")
        marker_style = self.console.get_style(
            "restructuredtext.blockquote_attribution_marker", default="bright_magenta"
        )
        author_style = self.console.get_style("restructuredtext.blockquote_attribution_text", default="grey89")
        children = list(node.children)
        attribution = children[-1] if children and isinstance(children[-1], docutils.nodes.attribution) else None
        paragraphs = children[:-1] if attribution else children

        for index, paragraph in enumerate(paragraphs):
            paragraph_text = paragraph.astext().replace("\n", " ")
            if index:
                self.renderables.append(NewLine())
                self.renderables.append(NewLine())
            self.renderables.append(Text("▌ ", style=marker_style) + Text(paragraph_text, style=text_style))

        if attribution:
            self.renderables.append(NewLine())
            self.renderables.append(Text("  - " + attribution.astext(), style=author_style))
        else:
            self.renderables.append(NewLine())
            self.renderables.append(NewLine())

        raise docutils.nodes.SkipChildren()

    def _render_line_block(self, node, indent=0):
        """Recursively render a line_block node, preserving nested indentation."""
        prefix = "    " * indent
        for child in node.children:
            if isinstance(child, docutils.nodes.line_block):
                self._render_line_block(child, indent + 1)
            elif isinstance(child, docutils.nodes.line):
                self.renderables.append(Text(prefix + child.astext()))

    def visit_line_block(self, node):
        self._render_line_block(node)
        raise docutils.nodes.SkipChildren()

    def _collect_body_renderables(self, children):
        """Render a list of body nodes into renderables, returning the collected list.

        Handles bullet_list, enumerated_list, and paragraph nodes explicitly;
        other node types fall back to plain text via ``astext()``.
        """
        saved = self.renderables
        self.renderables = []
        for child in children:
            if isinstance(child, docutils.nodes.bullet_list):
                self._render_bullet_list(child, level=0)
            elif isinstance(child, docutils.nodes.enumerated_list):
                self._render_enumerated_list(child, level=0)
            elif isinstance(child, docutils.nodes.paragraph):
                self.renderables.append(Text(child.astext().replace("\n", " ")))
            else:
                text = child.astext().replace("\n", " ")
                if text:
                    self.renderables.append(Text(text))
        body = self.renderables
        self.renderables = saved
        return body

    def visit_topic(self, node):
        style = self.console.get_style("restructuredtext.topic", default="bold cyan")
        children = list(node.children)
        title = ""
        body_start = 0
        if children and isinstance(children[0], docutils.nodes.title):
            title = children[0].astext()
            body_start = 1

        body_renderables = self._collect_body_renderables(children[body_start:])

        if body_renderables:
            self.renderables.append(
                Panel(Group(*body_renderables), title=title, style=style, border_style=style)
            )
        else:
            self.renderables.append(Panel("", title=title, style=style, border_style=style))
        raise docutils.nodes.SkipChildren()

    def visit_sidebar(self, node):
        children = list(node.children)
        title = children[0] if children else ""
        subtitle = ""
        body_children = children[1:]

        if body_children and isinstance(body_children[0], docutils.nodes.subtitle):
            subtitle = body_children[0].astext()
            body_children = body_children[1:]

        paragraph = "\n\n".join(child.astext() for child in body_children)

        self.renderables.append(Panel(paragraph, title=title.astext(), subtitle=subtitle, expand=False))

        raise docutils.nodes.SkipChildren()

    def visit_transition(self, node):
        style = self.console.get_style("restructuredtext.hr", default="yellow")
        self.renderables.append(Rule(style=style))

    def visit_math_block(self, node):
        if self.renderables and isinstance(self.renderables[-1], Text):
            self.renderables[-1].append_text(Text(node.astext(), end=" "))
            raise docutils.nodes.SkipChildren()
        self.renderables.append(Text(node.astext()))
        raise docutils.nodes.SkipChildren()

    def visit_citation(self, node):
        border_style = self.console.get_style("restructuredtext.citation_border", default="grey74")
        self.renderables.append(Panel(node.astext(), title="citation", border_style=border_style))
        raise docutils.nodes.SkipChildren()

    def visit_citation_reference(self, node):
        style = self.console.get_style("restructuredtext.citation_reference", default="grey74")
        if self.renderables and isinstance(self.renderables[-1], Text):
            self.renderables[-1].append(node.astext().replace("\n", " "), style=style)
            raise docutils.nodes.SkipChildren()
        self.renderables.append(Text(node.astext().replace("\n", " "), style=style, end=""))
        raise docutils.nodes.SkipChildren()

    def visit_header(self, node):
        style = self.console.get_style("restructuredtext.caption", default="bold")
        self.renderables.insert(0, Panel(Align(node.astext(), "center"), title="caption", box=box.DOUBLE, style=style))
        raise docutils.nodes.SkipChildren()

    def visit_footer(self, node):
        self.footer.append(Align(node.astext(), "center"))
        raise docutils.nodes.SkipChildren()

    def visit_footnote_reference(self, node):
        style = self.console.get_style("restructuredtext.footnote_reference", default="grey74")
        newline = '\n'
        text = f"[{node.astext().replace(newline, ' ')}]"
        if self.renderables and isinstance(self.renderables[-1], Text):
            self.renderables[-1].append(text, style=style)
            raise docutils.nodes.SkipChildren()
        self.renderables.append(Text(text, style=style, end=""))
        raise docutils.nodes.SkipChildren()

    def visit_substitution_reference(self, node):
        style = self.console.get_style("restructuredtext.substitution_reference", default="none")
        text = node.astext().replace("\n", " ")
        if self.renderables and isinstance(self.renderables[-1], Text):
            self.renderables[-1].append(text, style=style)
            raise docutils.nodes.SkipChildren()
        self.renderables.append(Text(text, style=style, end=""))
        raise docutils.nodes.SkipChildren()

    def visit_footnote(self, node):
        self.footer.append(Align(node.astext(), "left"))
        raise docutils.nodes.SkipChildren()

    def visit_generated(self, node):
        self.footer.append(Align(node.astext(), "left"))
        raise docutils.nodes.SkipChildren()

    def visit_pending(self, node):
        raise docutils.nodes.SkipChildren()

    def visit_problematic(self, node):
        self.errors.append(
            Panel(
                Syntax(node.astext(), lexer="rst", theme=self.code_theme),
                title=f"System Message: Problematic Element",
                border_style="bold red",
            ),
        )
        raise docutils.nodes.SkipChildren()

    def visit_raw(self, node):
        style = self.console.get_style("restructuredtext.literal_block_border", default="grey58")
        lexer = self._find_lexer(node)
        text = node.astext()
        title = ("stripped raw html" if lexer == "html" else "raw " + lexer)

        if lexer == "html":
            text = strip_tags(text)
            lexer = self._guess_lexer_name(text) if self.guess_lexer else self.default_lexer

        self.renderables.append(
            Panel(
                Syntax(text, lexer, theme=self.code_theme, line_numbers=self.show_line_numbers),
                border_style=style,
                box=box.SQUARE,
                title=title,
            )
        )
        raise docutils.nodes.SkipChildren()

    def visit_table(self, node):
        header_style = self.console.get_style("restructuredtext.table_header", default="bold")
        cell_style = self.console.get_style("restructuredtext.table_cell", default="none")

        # Extract optional caption/title and the tgroup
        title = None
        tgroup = None
        for child in node.children:
            if isinstance(child, (docutils.nodes.title, docutils.nodes.caption)):
                title = child.astext()
            elif isinstance(child, docutils.nodes.tgroup):
                tgroup = child

        if tgroup is None:
            raise docutils.nodes.SkipChildren()

        # Find thead and tbody within tgroup
        thead = None
        tbody = None
        for child in tgroup.children:
            if isinstance(child, docutils.nodes.thead):
                thead = child
            elif isinstance(child, docutils.nodes.tbody):
                tbody = child

        if tbody is None:
            raise docutils.nodes.SkipChildren()

        # Build the rich Table
        has_header = thead is not None and bool(thead.children)
        rich_table = Table(
            show_header=has_header,
            title=title,
            header_style=header_style,
            show_lines=True,
        )

        # Add columns, using header-row entries as column labels when thead exists
        if thead is not None and thead.children:
            for entry in thead.children[0].children:
                rich_table.add_column(entry.astext().replace("\n", " "), style=cell_style)
        elif tbody.children:
            for _ in tbody.children[0].children:
                rich_table.add_column("", style=cell_style)

        # Add body rows
        for row in tbody.children:
            rich_table.add_row(
                *[Text(entry.astext().replace("\n", " "), style=cell_style) for entry in row.children]
            )

        self.renderables.append(rich_table)
        raise docutils.nodes.SkipChildren()


class RestructuredText(JupyterMixin):
    """A reStructuredText renderable for rich.

    Parameters
    ----------
    markup : str
        A string containing reStructuredText markup.
    code_theme : Optional[Union[str, SyntaxTheme]]
        Pygments theme for code blocks. Defaults to "monokai".
    show_line_numbers : Optional[bool]
        Whether to display line numbers for syntax-highlighted code blocks.
    show_errors : Optional[bool]
        Whether to show system_messages aka errors and warnings.
    guess_lexer : Optional[bool]
        Whether to guess lexers for code blocks without specified language.
    default_lexer : Optional[str]
        Which lexer to use if no lexer is guessed or found. Defaults to "python"
    sphinx_compat : Optional[bool]
        Enable compatibility with Sphinx roles (func, meth, class, etc.) commonly used in
        Python docstrings. When enabled, these roles render as inline code instead of errors.
        Defaults to True for better compatibility with Python documentation.
    filename : Optional[str]
        A file name to use for error messages, useful for debugging purposes. Defaults to "<rst-document>"
    """

    def __init__(
        self,
        markup: str,
        code_theme: Optional[Union[str, SyntaxTheme]] = "monokai",
        show_line_numbers: Optional[bool] = False,
        show_errors: Optional[bool] = True,
        guess_lexer: Optional[bool] = False,
        default_lexer: Optional[str] = "python",
        sphinx_compat: Optional[bool] = True,
        filename: Optional[str] = "<rst-document>"
    ) -> None:
        self.markup = markup
        self.code_theme = code_theme
        self.show_line_numbers = show_line_numbers
        self.log_errors = show_errors
        self.guess_lexer = guess_lexer
        self.default_lexer = default_lexer
        self.sphinx_compat = sphinx_compat
        self.filename = filename

    def __rich_console__(self, console: Console, options: ConsoleOptions) -> RenderResult:
        if self.sphinx_compat:
            _register_sphinx_roles()
            _register_sphinx_directives()

        # Use the full docutils publish pipeline so that all standard transforms
        # (substitution resolution, hyperlink resolution, footnote numbering,
        # bibliographic-field promotion, …) are applied before we walk the tree.
        document = docutils.core.publish_doctree(
            self.markup,
            source_path=self.filename,
            settings_overrides={"report_level": 69, "halt_level": 69},
        )

        # Render the RST `document` using Rich.
        visitor = RSTVisitor(
            document,
            console=console,
            code_theme=self.code_theme,
            show_line_numbers=self.show_line_numbers,
            guess_lexer=self.guess_lexer,
            default_lexer=self.default_lexer,
        )
        document.walkabout(visitor)

        # Strip all trailing newlines and newline-like rich objects
        while visitor.renderables:
            if isinstance(visitor.renderables[-1], Text):
                visitor.renderables[-1].rstrip()
                visitor.renderables[-1].end = "\n"
                if visitor.renderables[-1]:  # The Text object still contains data.
                    break
                else:
                    visitor.renderables.pop()
            elif isinstance(visitor.renderables[-1], NewLine):
                visitor.renderables.pop()
            else:
                break

        for renderable in visitor.renderables:
            yield from console.render(renderable, options)
        if self.log_errors and visitor.errors:
            for error in visitor.errors:
                yield from console.render(error, options)
        style = console.get_style("restructuredtext.footer", default="none")
        border_style = console.get_style("restructuredtext.footer_border", default="grey74")
        if visitor.footer:
            yield from console.render(
                Panel(Group(*visitor.footer), title="Footer", box=box.SQUARE, border_style=border_style, style=style)
            )


RST = reST = ReStructuredText = reStructuredText = RestructuredText
